@@ -1,7 +1,8 @@
+#!/usr/bin/env python3
 #
 # Copyright (c) 2020      Intel, Inc.  All rights reserved.
 # Copyright (c) 2020-2022 Cisco Systems, Inc.  All rights reserved
-# Copyright (c) 2021-2023 Nanook Consulting  All rights reserved.
+# Copyright (c) 2021-2025 Nanook Consulting  All rights reserved.
 # Copyright (c) 2022      Triad National Security, LLC. All rights reserved.
 # $COPYRIGHT$
 #
@@ -10,11 +11,9 @@
 # by tools to interpret user input
 #
 
-from __future__ import print_function
 import os
-import os.path
 import sys
-from optparse import OptionParser, OptionGroup
+import argparse
 
 index = 0
 
@@ -239,7 +238,7 @@ def harvest_constants(options, path, constants):
     return 0
 
 
-def _write_header(options, base_path, num_elements):
+def _write_header(options, num_elements):
     contents = '''/*
  * This file is autogenerated by construct_dictionary.py.
  * Do not edit this file by hand.
@@ -267,7 +266,7 @@ END_C_DECLS
         constants = sys.stdout
         outpath = None
     else:
-        outpath = os.path.join(base_path, "pmix_dictionary.h")
+        outpath = os.path.join(options.out, "pmix_dictionary.h")
         try:
             constants = open(outpath, "w+")
         except Exception as e:
@@ -280,44 +279,26 @@ END_C_DECLS
 
 
 def main():
-    parser = OptionParser("usage: %prog [options]")
-    debugGroup = OptionGroup(parser, "Debug Options")
-    debugGroup.add_option("--dryrun",
-                          action="store_true", dest="dryrun", default=False,
-                          help="Show output to screen")
-    parser.add_option_group(debugGroup)
+    parser = argparse.ArgumentParser(description="Generate attribute dictionary.")
+    parser.add_argument("--root",
+                        required=True,
+                        help="Root directory of code tree")
+    parser.add_argument("--out",
+                        required=True,
+                        help="Output C file")
+    parser.add_argument("--dryrun",
+                        action="store_true",
+                        help="Do not write out resulting file")
 
-    (options, args) = parser.parse_args()
+    args = parser.parse_args()
 
-    # Find the top-level PMIx source tree dir.
-    # Start with the location of this script, which we know is in
-    # $top_srcdir/contrib.
-    top_src_dir = os.path.dirname(sys.argv[0])
-    top_src_dir = os.path.join(top_src_dir, "..")
-    top_src_dir = os.path.abspath(top_src_dir)
+    source_include_dir = os.path.join(args.root, "include")
 
-    # Sanity check
-    checkfile = os.path.join(top_src_dir, "VERSION")
-    if not os.path.exists(checkfile):
-        print("ERROR: Could not find top source directory for Open PMIx")
-        return 1
-
-    source_include_dir = os.path.join(top_src_dir, "include")
-
-    # This script is invoked from src/include/Makefile.am, and
-    # therefore the cwd will be $(builddir)/src/include.  Verify this
-    # by checking for a file that we know should be in there.
-    build_src_include_dir = os.getcwd()
-    checkfile = os.path.join(build_src_include_dir, "pmix_config.h")
-    if not os.path.exists(checkfile):
-        print("ERROR: Could not find build directory for Open PMIx")
-        return 1
-
-    if options.dryrun:
+    if args.dryrun:
         constants = sys.stdout
         outpath = None
     else:
-        outpath = os.path.join(build_src_include_dir, "pmix_dictionary.c")
+        outpath = os.path.join(args.out, "pmix_dictionary.c")
         try:
             constants = open(outpath, "w+")
         except Exception as e:
@@ -340,7 +321,7 @@ const pmix_regattr_input_t pmix_dictionary[] = {
     # looking for constants and typedefs
 
     # pmix_common.h.in is in the src tree
-    rc = harvest_constants(options,
+    rc = harvest_constants(args,
                            os.path.join(source_include_dir, "pmix_common.h.in"),
                            constants)
     if 0 != rc:
@@ -352,7 +333,7 @@ const pmix_regattr_input_t pmix_dictionary[] = {
     constants.write(",\n")
 
     # pmix_deprecated.h is in the source tree
-    rc = harvest_constants(options,
+    rc = harvest_constants(args,
                            os.path.join(source_include_dir, "pmix_deprecated.h"),
                            constants)
     if 0 != rc:
@@ -363,15 +344,12 @@ const pmix_regattr_input_t pmix_dictionary[] = {
         return 1
 
     # mark the end of the array
-    constants.write(""",\n
-    {.index = UINT32_MAX, .name = "", .string = "", .type = PMIX_POINTER, .description = (char *[]){"NONE", NULL}}
-};
-""")
+    constants.write("""\n};""")
     constants.write("\n")
     constants.close()
 
     # write the header
-    return _write_header(options, build_src_include_dir, index + 1)
+    return _write_header(args, index)
 
 
 if __name__ == '__main__':
