@@ -4,7 +4,7 @@
  *                         All rights reserved.
  * Copyright (c) 2015      Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
- * Copyright (c) 2021      Nanook Consulting.  All rights reserved.
+ * Copyright (c) 2021-2024 Nanook Consulting  All rights reserved.
  * $COPYRIGHT$
  *
  * Redistribution and use in source and binary forms, with or without
@@ -536,6 +536,21 @@ typedef pmix_status_t (*pmix_server_fabric_fn_t)(const pmix_proc_t *requestor,
                                                  const pmix_info_t directives[], size_t ndirs,
                                                  pmix_info_cbfunc_t cbfunc, void *cbdata);
 
+/* Execute a session control operation request */
+typedef pmix_status_t (*pmix_server_session_control_fn_t)(const pmix_proc_t *requestor,
+                                                          uint32_t sessionID,
+                                                          const pmix_info_t directives[], size_t ndirs,
+                                                          pmix_info_cbfunc_t cbfunc, void *cbdata);
+
+/* Define/delete or extend/remove resources from a resource block */
+typedef pmix_status_t (*pmix_server_resource_block_fn_t)(const pmix_proc_t *requestor,
+                                                         pmix_resource_block_directive_t directive,
+                                                         const char *block,
+                                                         const pmix_resource_unit_t *units, size_t nunit,
+                                                         const pmix_info_t *info, size_t ninfo,
+                                                         pmix_op_cbfunc_t cbfunc, void *cbdata);
+
+
 typedef struct pmix_server_module_4_0_0_t {
     /* v1x interfaces */
     pmix_server_client_connected_fn_t   client_connected;
@@ -569,6 +584,9 @@ typedef struct pmix_server_module_4_0_0_t {
     pmix_server_grp_fn_t                group;
     pmix_server_fabric_fn_t             fabric;
     pmix_server_client_connected2_fn_t  client_connected2;
+    /* v5x interfaces */
+    pmix_server_session_control_fn_t    session_control;
+    pmix_server_resource_block_fn_t     resource_block;
 } pmix_server_module_t;
 
 /****    HOST RM FUNCTIONS FOR INTERFACE TO PMIX SERVER    ****/
@@ -845,11 +863,19 @@ PMIX_EXPORT pmix_status_t PMIx_server_generate_locality_string(const pmix_cpuset
 PMIX_EXPORT pmix_status_t PMIx_server_generate_cpuset_string(const pmix_cpuset_t *cpuset,
                                                              char **cpuset_string);
 
+/* Generate a cpuset from the provided PMIx string representation.
+ * Provide a function by which the host environment can convert a PMIx
+ * string representation of a cpuset into the corresponding cpuset bitmap.
+ * Caller must provide storage for the pmix_cpuset_t.
+ */
+PMIX_EXPORT pmix_status_t PMIx_server_generate_cpuset(const char *cpuset_string,
+                                                      pmix_cpuset_t *cpuset);
+
 /* Define a process set
  * Provide a function by which the host environment can define a new process set.
  */
 PMIX_EXPORT pmix_status_t PMIx_server_define_process_set(const pmix_proc_t *members,
-                                                         size_t nmembers, char *pset_name);
+                                                         size_t nmembers, const char *pset_name);
 
 /* Delete a process set
  * Provide a function by which the host environment can delete a new process set.
@@ -858,6 +884,18 @@ PMIX_EXPORT pmix_status_t PMIx_server_delete_process_set(char *pset_name);
 
 
 /* Register non-namespace related information with the local PMIx server library.
+ * This API can be used for a variety of purposes. For example, one could use it
+ * to register information on the physical cluster that you want passed to every
+ * nspace during "register nspace".
+ *
+ * Another usage is to register data collected during various operations - e.g.,
+ * PMIx_Group_construct when returning group and endpoint information during a
+ * bootstrap operation. In such cases, not every local host will have at least
+ * one client involved in directly calling the construct API - i.e., all local
+ * "participants" may be doing so as "add members" and just waiting for an invite
+ * event. Thus, there is no callback function registered by the PMIx server
+ * library that can be used to store the returned endpoint, group info, and job
+ * data. This API fills that hole
  */
 PMIX_EXPORT pmix_status_t PMIx_server_register_resources(pmix_info_t info[], size_t ninfo,
                                                          pmix_op_cbfunc_t cbfunc,
@@ -869,6 +907,11 @@ PMIX_EXPORT pmix_status_t PMIx_server_deregister_resources(pmix_info_t info[], s
                                                            pmix_op_cbfunc_t cbfunc,
                                                            void *cbdata);
 
+/* collect job-level info for an array of procs */
+PMIX_EXPORT pmix_status_t PMIx_server_collect_job_info(pmix_proc_t *procs, size_t nprocs,
+                                                       pmix_data_buffer_t *dbuf);
+
+
 #if defined(c_plusplus) || defined(__cplusplus)
 }
 #endif
