@@ -8,7 +8,8 @@
  * Copyright (c) 2016      Mellanox Technologies, Inc.
  *                         All rights reserved.
  * Copyright (c) 2016      IBM Corporation.  All rights reserved.
- * Copyright (c) 2021-2023 Nanook Consulting  All rights reserved.
+ * Copyright (c) 2021-2025 Nanook Consulting  All rights reserved.
+ * Copyright (c) 2024      Triad National Security, LLC. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -54,6 +55,7 @@
 #include "src/util/pmix_argv.h"
 #include "src/util/pmix_error.h"
 #include "src/util/pmix_output.h"
+#include "src/util/pmix_printf.h"
 
 #include "pmix_client_ops.h"
 
@@ -80,7 +82,7 @@ static void gtcon(pmix_group_tracker_t *p)
 {
     PMIX_CONSTRUCT_LOCK(&p->lock);
     p->status = PMIX_SUCCESS;
-    p->ref = 0;
+    p->ref = SIZE_MAX;
     p->accepted = 0;
     p->grpid = NULL;
     p->members = NULL;
@@ -124,6 +126,166 @@ static void op_cbfunc_rel(pmix_status_t status, void *cbdata);
 
 static void info_cbfunc(pmix_status_t status, pmix_info_t *info, size_t ninfo, void *cbdata,
                         pmix_release_cbfunc_t release_fn, void *release_cbdata);
+static pmix_status_t add_group(const char *grpid,
+                               size_t ctxid,
+                               pmix_proc_t *members, size_t nmembers);
+
+static pmix_status_t get_endpts(pmix_info_t *xfer,
+                                pmix_scope_t scope,
+                                bool *endpts)
+{
+    pmix_cb_t cb2;
+    bool found;
+    pmix_kval_t *kv;
+    void *ilist;
+    pmix_status_t rc;
+    pmix_data_array_t darray;
+
+    PMIX_CONSTRUCT(&cb2, pmix_cb_t);
+    cb2.proc = &pmix_globals.myid;
+    cb2.scope = scope;
+    cb2.copy = true;
+    PMIX_GDS_FETCH_KV(rc, pmix_globals.mypeer, &cb2);
+    if (PMIX_SUCCESS == rc) {
+        ilist = PMIx_Info_list_start();
+        // start with our procID
+        PMIx_Info_list_add(ilist, PMIX_PROCID, &pmix_globals.myid, PMIX_PROC);
+        // add the scope
+        PMIx_Info_list_add(ilist, PMIX_DATA_SCOPE, &cb2.scope, PMIX_SCOPE);
+        // now add the kvals
+        found = false;
+        PMIX_LIST_FOREACH (kv, &cb2.kvs, pmix_kval_t) {
+            if (PMIx_Check_reserved_key(kv->key)) {
+                continue;
+            }
+            PMIx_Info_list_add_value_unique(ilist, kv->key, kv->value);
+            found = true;
+        }
+        if (found) {
+            // convert to array
+            rc = PMIx_Info_list_convert(ilist, &darray);
+            if (PMIX_SUCCESS != rc) {
+                PMIX_ERROR_LOG(rc);
+                PMIx_Info_list_release(ilist);
+                return rc;
+            }
+            // insert into a pmix_info_t for packing
+            PMIX_INFO_LOAD(xfer, PMIX_PROC_DATA, &darray, PMIX_DATA_ARRAY);
+            PMIX_DATA_ARRAY_DESTRUCT(&darray);
+            *endpts = true;
+        }
+        PMIx_Info_list_release(ilist);
+    }
+    PMIX_DESTRUCT(&cb2);
+    return PMIX_SUCCESS;
+}
+
+static pmix_status_t construct_msg(pmix_buffer_t *msg,
+                                   const char *grp,
+                                   const pmix_proc_t *procs, size_t nprocs,
+                                   const pmix_info_t *info, size_t ninfo)
+{
+    pmix_status_t rc;
+    pmix_cmd_t cmd = PMIX_GROUP_CONSTRUCT_CMD;
+    pmix_info_t local_endpts, *icopy, *iarray, *iptr;
+    size_t sz, n, m, niarray;
+    bool lclendpts;
+
+    /* pack the cmd */
+    PMIX_BFROPS_PACK(rc, pmix_client_globals.myserver, msg, &cmd, 1, PMIX_COMMAND);
+    if (PMIX_SUCCESS != rc) {
+        PMIX_ERROR_LOG(rc);
+        return rc;
+    }
+
+    /* pack the group ID */
+    PMIX_BFROPS_PACK(rc, pmix_client_globals.myserver, msg, &grp, 1, PMIX_STRING);
+    if (PMIX_SUCCESS != rc) {
+        PMIX_ERROR_LOG(rc);
+        return rc;
+    }
+
+    /* pack the number of procs */
+    PMIX_BFROPS_PACK(rc, pmix_client_globals.myserver, msg, &nprocs, 1, PMIX_SIZE);
+    if (PMIX_SUCCESS != rc) {
+        PMIX_ERROR_LOG(rc);
+        return rc;
+    }
+    if (0 < nprocs) {
+        PMIX_BFROPS_PACK(rc, pmix_client_globals.myserver, msg, procs, nprocs, PMIX_PROC);
+        if (PMIX_SUCCESS != rc) {
+            PMIX_ERROR_LOG(rc);
+            return rc;
+        }
+    }
+
+    /* get our endpt info, if some was posted. We use
+     * "remote" scope as all local procs have access
+     * to info posted by all other local procs, regardless
+     * of their namespace */
+    sz = ninfo;
+    lclendpts = false;
+    rc = get_endpts(&local_endpts, PMIX_REMOTE, &lclendpts);
+    if (PMIX_SUCCESS != rc) {
+        PMIX_ERROR_LOG(rc);
+        return rc;
+    }
+    if (lclendpts) {
+        sz = sz + 1;
+    }
+    PMIX_INFO_CREATE(icopy, sz);
+
+    // check for group info
+    for (n=0; n < ninfo; n++) {
+        if (PMIX_CHECK_KEY(&info[n], PMIX_GROUP_INFO)) {
+            iarray = (pmix_info_t*)info[n].value.data.darray->array;
+            niarray = info[n].value.data.darray->size;
+            // check if the first entry is our procID
+            if (PMIX_PROC != iarray[0].value.type) {
+                // we need to add our ID to the beginning of the array
+                PMIX_INFO_CREATE(iptr, niarray+1);
+                PMIX_INFO_LOAD(&iptr[0], PMIX_PROCID, &pmix_globals.myid, PMIX_PROC);
+                for (m=0; m < niarray; m++) {
+                    PMIX_INFO_XFER(&iptr[m+1], &iarray[m]);
+                }
+                PMIx_Load_key(icopy[n].key, PMIX_GROUP_INFO);
+                icopy[n].value.type = PMIX_DATA_ARRAY;
+                icopy[n].value.data.darray = (pmix_data_array_t*)pmix_malloc(sizeof(pmix_data_array_t));
+                icopy[n].value.data.darray->type = PMIX_INFO;
+                icopy[n].value.data.darray->array = iptr;
+                icopy[n].value.data.darray->size = niarray + 1;
+            } else {
+                PMIX_INFO_XFER(&icopy[n], &info[n]);
+            }
+        } else {
+            PMIX_INFO_XFER(&icopy[n], &info[n]);
+        }
+    }
+    if (lclendpts) {
+        // add the local endpt data
+        PMIX_INFO_XFER(&icopy[n], &local_endpts);
+        PMIX_INFO_DESTRUCT(&local_endpts);
+        ++n;
+    }
+
+    /* pack the info structs */
+    PMIX_BFROPS_PACK(rc, pmix_client_globals.myserver, msg, &sz, 1, PMIX_SIZE);
+    if (PMIX_SUCCESS != rc) {
+        PMIX_ERROR_LOG(rc);
+        PMIX_INFO_FREE(icopy, sz);
+        return rc;
+    }
+    if (0 < sz) {
+        PMIX_BFROPS_PACK(rc, pmix_client_globals.myserver, msg, icopy, sz, PMIX_INFO);
+        if (PMIX_SUCCESS != rc) {
+            PMIX_ERROR_LOG(rc);
+            return rc;
+        }
+        PMIX_INFO_FREE(icopy, sz);
+    }
+
+    return rc;
+}
 
 PMIX_EXPORT pmix_status_t PMIx_Group_construct(const char grp[], const pmix_proc_t procs[],
                                                size_t nprocs, const pmix_info_t info[],
@@ -135,7 +297,7 @@ PMIX_EXPORT pmix_status_t PMIx_Group_construct(const char grp[], const pmix_proc
 
     PMIX_ACQUIRE_THREAD(&pmix_global_lock);
 
-    pmix_output_verbose(2, pmix_client_globals.connect_output,
+    pmix_output_verbose(2, pmix_client_globals.group_output,
                         "pmix: group_construct called");
 
     if (pmix_globals.init_cntr <= 0) {
@@ -162,19 +324,18 @@ PMIX_EXPORT pmix_status_t PMIx_Group_construct(const char grp[], const pmix_proc
         return rc;
     }
 
-    /* wait for the connect to complete */
+    /* wait for the group construct to complete */
     PMIX_WAIT_THREAD(&cb->lock);
     rc = cb->status;
-    /* user takes responsibility for releasing any results */
-    *results = cb->results;
-    *nresults = cb->nresults;
-    cb->results = NULL;
-    cb->nresults = 0;
+    if (PMIX_SUCCESS == rc) {
+        /* user takes responsibility for releasing any results */
+        *results = cb->results;
+        *nresults = cb->nresults;
+        cb->results = NULL;
+        cb->nresults = 0;
+    }
     PMIX_RELEASE(cb);
 
-    pmix_output_verbose(2, pmix_globals.debug_output,
-                        "pmix: group construct completed");
-
     return rc;
 }
 
@@ -184,13 +345,12 @@ PMIX_EXPORT pmix_status_t PMIx_Group_construct_nb(const char grp[], const pmix_p
                                                   void *cbdata)
 {
     pmix_buffer_t *msg = NULL;
-    pmix_cmd_t cmd = PMIX_GROUP_CONSTRUCT_CMD;
     pmix_status_t rc;
     pmix_group_tracker_t *cb = NULL;
 
     PMIX_ACQUIRE_THREAD(&pmix_global_lock);
 
-    pmix_output_verbose(2, pmix_client_globals.connect_output,
+    pmix_output_verbose(2, pmix_client_globals.group_output,
                         "pmix:group_construct_nb called");
 
     if (pmix_globals.init_cntr <= 0) {
@@ -205,52 +365,12 @@ PMIX_EXPORT pmix_status_t PMIx_Group_construct_nb(const char grp[], const pmix_p
     }
     PMIX_RELEASE_THREAD(&pmix_global_lock);
 
-    /* check for bozo input */
-    if (NULL == procs || 0 >= nprocs) {
-        return PMIX_ERR_BAD_PARAM;
-    }
-
+    // send any data to our server
     msg = PMIX_NEW(pmix_buffer_t);
-    /* pack the cmd */
-    PMIX_BFROPS_PACK(rc, pmix_client_globals.myserver, msg, &cmd, 1, PMIX_COMMAND);
+    rc = construct_msg(msg, grp, procs, nprocs, info, ninfo);
     if (PMIX_SUCCESS != rc) {
         PMIX_ERROR_LOG(rc);
-        goto done;
-    }
-
-    /* pack the group ID */
-    PMIX_BFROPS_PACK(rc, pmix_client_globals.myserver, msg, &grp, 1, PMIX_STRING);
-    if (PMIX_SUCCESS != rc) {
-        PMIX_ERROR_LOG(rc);
-        goto done;
-    }
-
-    /* pack the number of procs */
-    PMIX_BFROPS_PACK(rc, pmix_client_globals.myserver, msg, &nprocs, 1, PMIX_SIZE);
-    if (PMIX_SUCCESS != rc) {
-        PMIX_ERROR_LOG(rc);
-        goto done;
-    }
-    PMIX_BFROPS_PACK(rc, pmix_client_globals.myserver, msg, procs, nprocs, PMIX_PROC);
-    if (PMIX_SUCCESS != rc) {
-        PMIX_ERROR_LOG(rc);
-        goto done;
-    }
-
-    /* pack the info structs */
-    PMIX_BFROPS_PACK(rc, pmix_client_globals.myserver, msg, &ninfo, 1, PMIX_SIZE);
-    if (PMIX_SUCCESS != rc) {
-        PMIX_ERROR_LOG(rc);
-        PMIX_RELEASE(msg);
-        goto done;
-    }
-    if (0 < ninfo) {
-        PMIX_BFROPS_PACK(rc, pmix_client_globals.myserver, msg, info, ninfo, PMIX_INFO);
-        if (PMIX_SUCCESS != rc) {
-            PMIX_ERROR_LOG(rc);
-            PMIX_RELEASE(msg);
-            goto done;
-        }
+        return rc;
     }
 
     /* create a callback object as we need to pass it to the
@@ -262,15 +382,13 @@ PMIX_EXPORT pmix_status_t PMIx_Group_construct_nb(const char grp[], const pmix_p
     cb->grpid = strdup(grp);
 
     /* push the message into our event base to send to the server */
-    PMIX_PTL_SEND_RECV(rc, pmix_client_globals.myserver, msg, construct_cbfunc, (void *) cb);
+    PMIX_PTL_SEND_RECV(rc, pmix_client_globals.myserver, msg, construct_cbfunc, (void*)cb);
     if (PMIX_SUCCESS != rc) {
+        PMIX_ERROR_LOG(rc);
         PMIX_RELEASE(cb);
-    }
-
-done:
-    if (PMIX_SUCCESS != rc && NULL != msg) {
         PMIX_RELEASE(msg);
     }
+
     return rc;
 }
 
@@ -283,7 +401,7 @@ PMIX_EXPORT pmix_status_t PMIx_Group_destruct(const char grp[],
 
     PMIX_ACQUIRE_THREAD(&pmix_global_lock);
 
-    pmix_output_verbose(2, pmix_client_globals.connect_output,
+    pmix_output_verbose(2, pmix_client_globals.group_output,
                         "pmix: group_destruct called");
 
     if (pmix_globals.init_cntr <= 0) {
@@ -315,7 +433,7 @@ PMIX_EXPORT pmix_status_t PMIx_Group_destruct(const char grp[],
     rc = cb.status;
     PMIX_DESTRUCT(&cb);
 
-    pmix_output_verbose(2, pmix_client_globals.connect_output,
+    pmix_output_verbose(2, pmix_client_globals.group_output,
                         "pmix: group destruct completed");
 
     return rc;
@@ -333,7 +451,7 @@ PMIX_EXPORT pmix_status_t PMIx_Group_destruct_nb(const char grpid[], const pmix_
 
     PMIX_ACQUIRE_THREAD(&pmix_global_lock);
 
-    pmix_output_verbose(2, pmix_client_globals.connect_output,
+    pmix_output_verbose(2, pmix_client_globals.group_output,
                         "pmix:group_destruct_nb called");
 
     if (pmix_globals.init_cntr <= 0) {
@@ -457,8 +575,8 @@ static void invite_handler(size_t evhdlr_registration_id, pmix_status_t status,
     pmix_proc_t *affected = NULL;
     size_t n;
     pmix_status_t rc = PMIX_GROUP_INVITE_DECLINED;
-    size_t contextid = SIZE_MAX;
-
+    size_t contextid;
+    bool gotctxid = false;
     PMIX_HIDE_UNUSED_PARAMS(evhdlr_registration_id, source, results, nresults);
 
     /* find the object we asked to be returned with event */
@@ -468,13 +586,16 @@ static void invite_handler(size_t evhdlr_registration_id, pmix_status_t status,
                 /* this is an unrecoverable error - need to abort */
             }
             cb = (pmix_group_tracker_t *) info[n].value.data.ptr;
+
         } else if (PMIX_CHECK_KEY(&info[n], PMIX_EVENT_AFFECTED_PROC)) {
             if (PMIX_PROC != info[n].value.type) {
                 /* this is an unrecoverable error - need to abort */
             }
             affected = info[n].value.data.proc;
+
         } else if (PMIX_CHECK_KEY(&info[n], PMIX_GROUP_CONTEXT_ID)) {
             PMIX_VALUE_GET_NUMBER(rc, &info[n].value, contextid, size_t);
+            gotctxid = true;
         }
     }
     if (NULL == cb) {
@@ -497,10 +618,16 @@ static void invite_handler(size_t evhdlr_registration_id, pmix_status_t status,
     /* if the reporting process terminated, then issue the corresponding
      * group event - it only goes to this process */
     if (PMIX_PROC_TERMINATED == status) {
-        cb->ninfo = 2;
+        if (gotctxid) {
+            cb->ninfo = 2;
+        } else {
+            cb->ninfo = 1;
+        }
         PMIX_INFO_CREATE(cb->info, cb->ninfo);
         PMIX_INFO_LOAD(&cb->info[0], PMIX_EVENT_AFFECTED_PROC, affected, PMIX_PROC);
-        PMIX_INFO_LOAD(&cb->info[1], PMIX_GROUP_CONTEXT_ID, &contextid, PMIX_SIZE);
+        if (gotctxid) {
+            PMIX_INFO_LOAD(&cb->info[1], PMIX_GROUP_CONTEXT_ID, &contextid, PMIX_SIZE);
+        }
         rc = PMIx_Notify_event(PMIX_GROUP_INVITE_FAILED, &pmix_globals.myid, PMIX_RANGE_PROC_LOCAL,
                                cb->info, cb->ninfo, chaincbfunc, (void *) cb);
         if (PMIX_SUCCESS != rc) {
@@ -522,7 +649,7 @@ complete:
     if (cb->accepted == cb->nmembers) {
         /* execute the completion callback */
         if (NULL != cb->cbfunc) {
-            cb->cbfunc(PMIX_SUCCESS, NULL, 0, cb->cbdata, relcbfunc, cb->cbdata);
+            cb->cbfunc(PMIX_SUCCESS, info, ninfo, cb->cbdata, relcbfunc, cb->cbdata);
         }
     }
 
@@ -547,6 +674,7 @@ PMIX_EXPORT pmix_status_t PMIx_Group_invite(const char grp[], const pmix_proc_t
     pmix_group_tracker_t *cb;
     pmix_status_t rc;
     size_t n;
+    pmix_data_array_t darray;
 
     PMIX_ACQUIRE_THREAD(&pmix_global_lock);
     if (pmix_globals.init_cntr <= 0) {
@@ -590,22 +718,21 @@ PMIX_EXPORT pmix_status_t PMIx_Group_invite(const char grp[], const pmix_proc_t
      * members - only sent to members. Servers will intercept
      * and update their membership lists */
     cb = PMIX_NEW(pmix_group_tracker_t);
-    PMIX_INFO_CREATE(cb->info, 3);
+    cb->ninfo = 4;
+    PMIX_INFO_CREATE(cb->info, cb->ninfo);
     if (NULL == cb->info) {
         PMIX_RELEASE(cb);
         return PMIX_ERR_NOMEM;
     }
-    cb->ninfo = 3;
     n = 0;
-    (void) strncpy(cb->info[n].key, PMIX_EVENT_CUSTOM_RANGE, PMIX_MAX_KEYLEN);
-    cb->info[n].value.type = PMIX_DATA_ARRAY;
-    PMIX_DATA_ARRAY_CREATE(cb->info[n].value.data.darray, nprocs, PMIX_PROC);
-    if (NULL == cb->info[n].value.data.darray ||
-        NULL == cb->info[n].value.data.darray->array) {
-        PMIX_RELEASE(cb);
-        return PMIX_ERR_NOMEM;
-    }
-    memcpy(cb->info[n].value.data.darray->array, procs, nprocs * sizeof(pmix_proc_t));
+    darray.type = PMIX_PROC;
+    darray.array = (pmix_proc_t*)procs;
+    darray.size = nprocs;
+    // load the custom range
+    PMIX_INFO_LOAD(&cb->info[n], PMIX_EVENT_CUSTOM_RANGE, &darray, PMIX_DATA_ARRAY);
+    ++n;
+    // provide final group membership
+    PMIX_INFO_LOAD(&cb->info[n], PMIX_GROUP_MEMBERSHIP, &darray, PMIX_DATA_ARRAY);
     ++n;
     /* mark that this only goes to non-default handlers */
     PMIX_INFO_LOAD(&cb->info[n], PMIX_EVENT_NON_DEFAULT, NULL, PMIX_BOOL);
@@ -825,7 +952,7 @@ PMIX_EXPORT pmix_status_t PMIx_Group_join(const char grp[], const pmix_proc_t *l
     rc = cb->status;
     PMIX_RELEASE(cb);
 
-    pmix_output_verbose(2, pmix_client_globals.connect_output,
+    pmix_output_verbose(2, pmix_client_globals.group_output,
                         "pmix: group construction completed");
 
     return rc;
@@ -842,10 +969,11 @@ PMIX_EXPORT pmix_status_t PMIx_Group_join_nb(const char grp[], const pmix_proc_t
     pmix_data_range_t range;
     PMIX_HIDE_UNUSED_PARAMS(grp, cbfunc);
 
-    pmix_output_verbose(2, pmix_client_globals.connect_output,
+    pmix_output_verbose(2, pmix_client_globals.group_output,
                         "[%s:%d] pmix: join nb called",
                         pmix_globals.myid.nspace, pmix_globals.myid.rank);
 
+    PMIX_ACQUIRE_THREAD(&pmix_global_lock);
     if (pmix_globals.init_cntr <= 0) {
         PMIX_RELEASE_THREAD(&pmix_global_lock);
         return PMIX_ERR_INIT;
@@ -902,7 +1030,7 @@ PMIX_EXPORT pmix_status_t PMIx_Group_join_nb(const char grp[], const pmix_proc_t
     if (PMIX_SUCCESS != rc) {
         PMIX_RELEASE(cb);
     }
-    pmix_output_verbose(2, pmix_client_globals.connect_output,
+    pmix_output_verbose(2, pmix_client_globals.group_output,
                         "[%s:%d] pmix: group invite %s",
                         pmix_globals.myid.nspace, pmix_globals.myid.rank,
                         (PMIX_GROUP_INVITE_ACCEPTED == code) ? "ACCEPTED" : "DECLINED");
@@ -918,7 +1046,7 @@ PMIX_EXPORT pmix_status_t PMIx_Group_leave(const char grp[],
 
     PMIX_ACQUIRE_THREAD(&pmix_global_lock);
 
-    pmix_output_verbose(2, pmix_client_globals.connect_output, "pmix: group_leave called");
+    pmix_output_verbose(2, pmix_client_globals.group_output, "pmix: group_leave called");
 
     if (pmix_globals.init_cntr <= 0) {
         PMIX_RELEASE_THREAD(&pmix_global_lock);
@@ -944,12 +1072,12 @@ PMIX_EXPORT pmix_status_t PMIx_Group_leave(const char grp[],
         return rc;
     }
 
-    /* wait for the connect to complete */
+    /* wait for the operation to complete */
     PMIX_WAIT_THREAD(&cb.lock);
     rc = cb.status;
     PMIX_DESTRUCT(&cb);
 
-    pmix_output_verbose(2, pmix_client_globals.connect_output,
+    pmix_output_verbose(2, pmix_client_globals.group_output,
                         "pmix: group leave completed");
 
     return rc;
@@ -966,7 +1094,7 @@ PMIX_EXPORT pmix_status_t PMIx_Group_leave_nb(const char grp[],
 
     PMIX_ACQUIRE_THREAD(&pmix_global_lock);
 
-    pmix_output_verbose(2, pmix_client_globals.connect_output,
+    pmix_output_verbose(2, pmix_client_globals.group_output,
                         "pmix:group_leave_nb called");
 
     if (pmix_globals.init_cntr <= 0) {
@@ -1061,32 +1189,33 @@ static void op_cbfunc_rel(pmix_status_t status, void *cbdata)
 
     cb->status = status;
     if (NULL != cb->cbfunc) {
-        cb->cbfunc(status, cb->info, cb->ninfo, cb->cbdata, NULL, NULL);
+        cb->cbfunc(status, cb->info, cb->ninfo, cb->cbdata, relfn, cb);
+    } else {
+        PMIX_RELEASE(cb);
     }
-    PMIX_RELEASE(cb);
 }
 
 static void construct_cbfunc(struct pmix_peer_t *pr,
-                       pmix_ptl_hdr_t *hdr,
-                       pmix_buffer_t *buf,
-                       void *cbdata)
+                             pmix_ptl_hdr_t *hdr,
+                             pmix_buffer_t *buf,
+                             void *cbdata)
 {
     pmix_group_tracker_t *cb = (pmix_group_tracker_t *) cbdata;
     pmix_status_t rc;
     pmix_status_t ret;
     int32_t cnt;
-    size_t ctxid, ninfo = 0, n;
-    pmix_info_t *iptr = NULL;
-    pmix_group_t *grp = NULL;
+    size_t ctxid = SIZE_MAX;
     bool gotctxid = false;
-    pmix_data_array_t darray;
     pmix_proc_t *members = NULL;
     size_t nmembers = 0;
+    pmix_data_array_t darray;
+    void *ilist;
 
     PMIX_HIDE_UNUSED_PARAMS(pr, hdr);
 
-    pmix_output_verbose(2, pmix_client_globals.connect_output,
-                        "pmix:client recv callback activated with %d bytes",
+    pmix_output_verbose(2, pmix_client_globals.group_output,
+                        "%s pmix:client construct callback activated with %d bytes",
+                        PMIX_NAME_PRINT(&pmix_globals.myid),
                         (NULL == buf) ? -1 : (int) buf->bytes_used);
 
     if (NULL == buf) {
@@ -1108,14 +1237,22 @@ static void construct_cbfunc(struct pmix_peer_t *pr,
         PMIX_ERROR_LOG(rc);
         ret = rc;
     }
+
+    if (PMIX_SUCCESS != ret) {
+        goto report;
+    }
+
     /* unpack the final membership */
     cnt = 1;
     PMIX_BFROPS_UNPACK(rc, pmix_client_globals.myserver, buf, &nmembers, &cnt, PMIX_SIZE);
-    if (PMIX_SUCCESS != rc && PMIX_ERR_UNPACK_READ_PAST_END_OF_BUFFER != rc) {
+    if (PMIX_ERR_UNPACK_READ_PAST_END_OF_BUFFER == rc) {
+        goto report;
+    } else if (PMIX_SUCCESS != rc) {
         PMIX_ERROR_LOG(rc);
         ret = rc;
         goto report;
-    } else if (PMIX_SUCCESS == rc) {
+    }
+    if (0 < nmembers) {
         PMIX_PROC_CREATE(members, nmembers);
         cnt = nmembers;
         PMIX_BFROPS_UNPACK(rc, pmix_client_globals.myserver, buf, members, &cnt, PMIX_PROC);
@@ -1126,60 +1263,63 @@ static void construct_cbfunc(struct pmix_peer_t *pr,
         }
     }
 
-    /* unpack any ctxid that was provided - it is okay if
-     * this attempts to unpack past end of buffer */
+    /* unpack any ctxid that was provided */
     cnt = 1;
-    PMIX_BFROPS_UNPACK(rc, pmix_client_globals.myserver, buf, &ctxid, &cnt, PMIX_SIZE);
-    if (PMIX_SUCCESS != rc && PMIX_ERR_UNPACK_READ_PAST_END_OF_BUFFER != rc) {
+    PMIX_BFROPS_UNPACK(rc, pmix_client_globals.myserver, buf, &gotctxid, &cnt, PMIX_BOOL);
+    if (PMIX_ERR_UNPACK_READ_PAST_END_OF_BUFFER == rc) {
+        goto report;
+    } else if (PMIX_SUCCESS != rc) {
         PMIX_ERROR_LOG(rc);
         ret = rc;
         goto report;
-    } else if (PMIX_SUCCESS == rc) {
-        gotctxid = true;
-    }
-
-
-    /* since the group construction has finished, we can add
-     * the group to out list of groups. Always sort the
-     * the array to maintain the same view across participants.*/
-    grp = PMIX_NEW(pmix_group_t);
-    PMIX_PROC_CREATE(grp->members, nmembers);
-    memcpy(grp->members, members, nmembers * sizeof(pmix_proc_t));
-    qsort(grp->members, nmembers, sizeof(pmix_proc_t), pmix_util_compare_proc);
-    grp->nmbrs = nmembers;
-    grp->grpid = strdup(cb->grpid);
-    pmix_list_append(&pmix_client_globals.groups, &grp->super);
-    darray.array = grp->members;
-    darray.size = grp->nmbrs;
-    darray.type = PMIX_PROC;
-    ++ninfo;
-
-    ++ninfo; // account for the grpID
-
-    if (gotctxid) {
-        ++ninfo;
-    }
-
-    PMIX_INFO_CREATE(iptr, ninfo);
-    n = 0;
-    PMIX_INFO_LOAD(&iptr[n], PMIX_GROUP_ID, cb->grpid, PMIX_STRING);
-    ++n;
-    if (0 < nmembers) {
-        PMIX_INFO_LOAD(&iptr[n], PMIX_GROUP_MEMBERSHIP, &darray, PMIX_DATA_ARRAY);
-        // do not destruct darray as the membership is being used
-        ++n;
     }
     if (gotctxid) {
-        PMIX_INFO_LOAD(&iptr[n], PMIX_GROUP_CONTEXT_ID, &ctxid, PMIX_SIZE);
-        ++n;
+        cnt = 1;
+        PMIX_BFROPS_UNPACK(rc, pmix_client_globals.myserver, buf, &ctxid, &cnt, PMIX_SIZE);
+        if (PMIX_SUCCESS != rc) {
+            PMIX_ERROR_LOG(rc);
+            ret = rc;
+        }
     }
 
+
 report:
+    ilist = PMIx_Info_list_start();
+
+    rc = PMIx_Info_list_add(ilist, PMIX_GROUP_ID, cb->grpid, PMIX_STRING);
+    if (PMIX_SUCCESS != rc) {
+        PMIx_Info_list_release(ilist);
+        goto done;
+    }
+    if (NULL != members) {
+        darray.array = members;
+        darray.size = nmembers;
+        darray.type = PMIX_PROC;
+        rc = PMIx_Info_list_add(ilist, PMIX_GROUP_MEMBERSHIP, &darray, PMIX_DATA_ARRAY);
+        // data was copied into the info
+        if (PMIX_SUCCESS != rc) {
+            PMIx_Info_list_release(ilist);
+            goto done;
+        }
+    }
+    if (gotctxid) {
+        rc = PMIx_Info_list_add(ilist, PMIX_GROUP_CONTEXT_ID, &ctxid, PMIX_SIZE);
+        if (PMIX_SUCCESS != rc) {
+            PMIx_Info_list_release(ilist);
+            goto done;
+        }
+    }
+    PMIx_Info_list_convert(ilist, &darray);
+    cb->info = (pmix_info_t*)darray.array;
+    cb->ninfo = darray.size;
+    PMIx_Info_list_release(ilist);
+
+done:
     if (NULL != members) {
         PMIX_PROC_FREE(members, nmembers);
     }
     if (NULL != cb->cbfunc) {
-        cb->cbfunc(ret, iptr, ninfo, cb->cbdata, relfn, cb);
+        cb->cbfunc(ret, cb->info, cb->ninfo, cb->cbdata, relfn, cb);
         return;
     }
     PMIX_RELEASE(cb);
@@ -1198,7 +1338,7 @@ static void destruct_cbfunc(struct pmix_peer_t *pr,
 
     PMIX_HIDE_UNUSED_PARAMS(pr, hdr);
 
-    pmix_output_verbose(2, pmix_client_globals.connect_output,
+    pmix_output_verbose(2, pmix_client_globals.group_output,
                         "pmix:client recv callback activated with %d bytes",
                         (NULL == buf) ? -1 : (int) buf->bytes_used);
 
@@ -1244,7 +1384,11 @@ static void info_cbfunc(pmix_status_t status, pmix_info_t *info, size_t ninfo, v
                         pmix_release_cbfunc_t release_fn, void *release_cbdata)
 {
     pmix_group_tracker_t *cb = (pmix_group_tracker_t *) cbdata;
-    size_t n;
+    size_t n, nmembers = 0;
+    pmix_proc_t *members = NULL;
+    char *grpid = NULL;
+    size_t ctxid = SIZE_MAX;
+    pmix_status_t rc;
 
     /* see if anything was returned - e.g., a context id */
     cb->status = status;
@@ -1253,12 +1397,51 @@ static void info_cbfunc(pmix_status_t status, pmix_info_t *info, size_t ninfo, v
         cb->nresults = ninfo;
         PMIX_INFO_CREATE(cb->results, cb->nresults);
         for (n = 0; n < ninfo; n++) {
+            if (PMIX_CHECK_KEY(&info[n], PMIX_GROUP_MEMBERSHIP)) {
+                members = (pmix_proc_t*)info[n].value.data.darray->array;
+                nmembers = info[n].value.data.darray->size;
+
+            } else if (PMIX_CHECK_KEY(&info[n], PMIX_GROUP_ID)) {
+                grpid = info[n].value.data.string;
+
+            } else if (PMIX_CHECK_KEY(&info[n], PMIX_GROUP_CONTEXT_ID)) {
+                PMIX_VALUE_GET_NUMBER(rc, &info[n].value, ctxid, size_t);
+                if (PMIX_SUCCESS != rc) {
+                    PMIX_ERROR_LOG(rc);
+                }
+            }
             PMIX_INFO_XFER(&cb->results[n], &info[n]);
         }
     }
+    if (NULL != members && NULL != grpid) {
+        add_group(grpid, ctxid, members, nmembers);
+    }
+
     if (NULL != release_fn) {
         release_fn(release_cbdata);
     }
     PMIX_POST_OBJECT(cb);
     PMIX_WAKEUP_THREAD(&cb->lock);
 }
+
+static pmix_status_t add_group(const char *grpid,
+                               size_t ctxid,
+                               pmix_proc_t *members, size_t nmembers)
+{
+    pmix_group_t *grp;
+
+    /* since the group construction has finished, we can add
+     * the group to out list of groups. Always sort the
+     * the array to maintain the same view across participants.*/
+    grp = PMIX_NEW(pmix_group_t);
+    PMIX_PROC_CREATE(grp->members, nmembers);
+    memcpy(grp->members, members, nmembers * sizeof(pmix_proc_t));
+    qsort(grp->members, nmembers, sizeof(pmix_proc_t), pmix_util_compare_proc);
+    grp->nmbrs = nmembers;
+    grp->grpid = strdup(grpid);
+    grp->ctxid = ctxid;
+    pmix_list_append(&pmix_client_globals.groups, &grp->super);
+
+    return PMIX_SUCCESS;
+}
+
