@@ -14,7 +14,7 @@
  * Copyright (c) 2014-2020 Intel, Inc.  All rights reserved.
  * Copyright (c) 2018      Research Organization for Information Science
  *                         and Technology (RIST).  All rights reserved.
- * Copyright (c) 2021-2023 Nanook Consulting.  All rights reserved.
+ * Copyright (c) 2021-2025 Nanook Consulting  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -203,6 +203,22 @@ PMIX_EXPORT const char *PMIx_Alloc_directive_string(pmix_alloc_directive_t direc
     }
 }
 
+PMIX_EXPORT const char *PMIx_Resource_block_directive_string(pmix_resource_block_directive_t directive)
+{
+    switch (directive) {
+    case PMIX_RESOURCE_BLOCK_DEFINE:
+        return "DEFINE";
+    case PMIX_RESOURCE_BLOCK_EXTEND:
+        return "EXTEND";
+    case PMIX_RESOURCE_BLOCK_REMOVE:
+        return "REMOVE";
+    case PMIX_RESOURCE_BLOCK_DELETE:
+        return "DELETE";
+    default:
+        return "UNSPECIFIED";
+    }
+}
+
 PMIX_EXPORT const char *pmix_command_string(pmix_cmd_t cmd)
 {
     switch (cmd) {
@@ -270,6 +286,16 @@ PMIX_EXPORT const char *pmix_command_string(pmix_cmd_t cmd)
         return "COMPUTE DEVICE DIST";
     case PMIX_REFRESH_CACHE:
         return "REFRESH CACHE";
+    case PMIX_RESBLK_CMD:
+        return "RESOURCE BLOCK";
+    case PMIX_SESSION_CTRL_CMD:
+        return "SESSION CONTROL";
+    case PMIX_REQ_SYSINFO_CMD:
+        return "REQUEST SYSTEM INFO";
+    case PMIX_RESOLVE_PEERS_CMD:
+        return "RESOLVE PEERS";
+    case PMIX_RESOLVE_NODE_CMD:
+        return "RESOLVE NODE";
     default:
         return "UNKNOWN";
     }
@@ -473,3 +499,25 @@ char* PMIx_Proc_string(const pmix_proc_t *proc)
     result = strdup(tmp);
     return result;
 }
+
+char* PMIx_Resource_unit_string(const pmix_resource_unit_t *p)
+{
+    char *tmp;
+
+    pmix_asprintf(&tmp, "TYPE: %s  COUNT: %" PRIsize_t "", PMIx_Device_type_string(p->type), p->count);
+    return tmp;
+}
+
+const char* PMIx_Group_operation_string(pmix_group_operation_t op)
+{
+    switch(op) {
+        case PMIX_GROUP_CONSTRUCT:
+            return "CONSTRUCT";
+        case PMIX_GROUP_DESTRUCT:
+            return "DESTRUCT";
+        case PMIX_GROUP_NONE:
+            return "NONE";
+        default:
+            return "UNKNOWN VALUE";
+    }
+}
