@@ -3,7 +3,7 @@
  * Copyright (c) 2014-2020 Intel, Inc.  All rights reserved.
  * Copyright (c) 2017-2018 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
- * Copyright (c) 2021-2022 Nanook Consulting.  All rights reserved.
+ * Copyright (c) 2021-2024 Nanook Consulting  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -12,7 +12,7 @@
  */
 #include "src/include/pmix_config.h"
 
-#include "pmix.h"
+#include "include/pmix.h"
 #include "pmix_common.h"
 #include "include/pmix_server.h"
 
@@ -77,7 +77,9 @@ static void rsdes(pmix_rshift_caddy_t *p)
         PMIX_RELEASE(p->cd);
     }
 }
-PMIX_CLASS_INSTANCE(pmix_rshift_caddy_t, pmix_object_t, rscon, rsdes);
+PMIX_CLASS_INSTANCE(pmix_rshift_caddy_t,
+                    pmix_object_t,
+                    rscon, rsdes);
 
 static void check_cached_events(pmix_rshift_caddy_t *cd);
 
@@ -102,7 +104,7 @@ static void regevents_cbfunc(struct pmix_peer_t *peer,
     cnt = 1;
     PMIX_BFROPS_UNPACK(rc, peer, buf, &ret, &cnt, PMIX_STATUS);
     if ((PMIX_SUCCESS != rc) || (PMIX_SUCCESS != ret)) {
-        /* remove the err handler and call the pmix_error.handler
+        /* remove the err handler and call the error handler
          * reg completion callback fn so the requestor
          * doesn't hang */
         if (NULL == rb->list) {
@@ -252,7 +254,8 @@ static pmix_status_t _add_hdlr(pmix_rshift_caddy_t *cd, pmix_list_t *xfer)
     if (NULL == cd->codes) {
         registered = false;
         PMIX_LIST_FOREACH (active, &pmix_globals.events.actives, pmix_active_code_t) {
-            if (PMIX_MAX_ERR_CONSTANT == active->code) {
+            if (PMIX_MAX_ERR_CONSTANT == active->code &&
+                (NULL != active->peer && active->peer == pmix_client_globals.myserver)) {
                 /* we have registered a default */
                 registered = true;
                 ++active->nregs;
@@ -263,6 +266,7 @@ static pmix_status_t _add_hdlr(pmix_rshift_caddy_t *cd, pmix_list_t *xfer)
             active = PMIX_NEW(pmix_active_code_t);
             active->code = PMIX_MAX_ERR_CONSTANT;
             active->nregs = 1;
+            active->peer = pmix_client_globals.myserver;
             pmix_list_append(&pmix_globals.events.actives, &active->super);
             /* ensure we register it */
             need_register = true;
@@ -271,7 +275,8 @@ static pmix_status_t _add_hdlr(pmix_rshift_caddy_t *cd, pmix_list_t *xfer)
         for (n = 0; n < cd->ncodes; n++) {
             registered = false;
             PMIX_LIST_FOREACH (active, &pmix_globals.events.actives, pmix_active_code_t) {
-                if (active->code == cd->codes[n]) {
+                if (active->code == cd->codes[n] &&
+                    (NULL != active->peer && active->peer == pmix_client_globals.myserver)) {
                     registered = true;
                     ++active->nregs;
                     break;
@@ -281,6 +286,7 @@ static pmix_status_t _add_hdlr(pmix_rshift_caddy_t *cd, pmix_list_t *xfer)
                 active = PMIX_NEW(pmix_active_code_t);
                 active->code = cd->codes[n];
                 active->nregs = 1;
+                active->peer = pmix_client_globals.myserver;
                 pmix_list_append(&pmix_globals.events.actives, &active->super);
                 /* ensure we register it */
                 need_register = true;
@@ -310,8 +316,10 @@ static pmix_status_t _add_hdlr(pmix_rshift_caddy_t *cd, pmix_list_t *xfer)
      * type with our server, or if we have directives, then we need to notify
      * the server - however, don't do this for a v1 server as the event
      * notification system there doesn't work */
-    if ((!PMIX_PEER_IS_SERVER(pmix_globals.mypeer) || PMIX_PEER_IS_LAUNCHER(pmix_globals.mypeer))
-        && pmix_globals.connected && !PMIX_PEER_IS_V1(pmix_client_globals.myserver)
+    if ((!PMIX_PEER_IS_SERVER(pmix_globals.mypeer) ||
+          PMIX_PEER_IS_LAUNCHER(pmix_globals.mypeer) ||
+          PMIX_PEER_IS_TOOL(pmix_globals.mypeer)) &&
+        pmix_globals.connected && !PMIX_PEER_IS_V1(pmix_client_globals.myserver)
         && (need_register || 0 < pmix_list_get_size(xfer))) {
         pmix_output_verbose(2, pmix_client_globals.event_output,
                             "pmix: _add_hdlr sending to server");
@@ -417,11 +425,9 @@ static void check_cached_events(pmix_rshift_caddy_t *cd)
             /* need to copy the info */
             for (n = 0; n < ncd->ninfo; n++) {
                 PMIX_INFO_XFER(&chain->info[n], &ncd->info[n]);
-                if (0 == strncmp(ncd->info[n].key, PMIX_EVENT_NON_DEFAULT, PMIX_MAX_KEYLEN)) {
+                if (PMIX_CHECK_KEY(&ncd->info[n], PMIX_EVENT_NON_DEFAULT)) {
                     chain->nondefault = true;
-                } else if (0
-                           == strncmp(ncd->info[n].key, PMIX_EVENT_AFFECTED_PROC,
-                                      PMIX_MAX_KEYLEN)) {
+                } else if (PMIX_CHECK_KEY(&ncd->info[n], PMIX_EVENT_AFFECTED_PROC)) {
                     PMIX_PROC_CREATE(chain->affected, 1);
                     if (NULL == chain->affected) {
                         PMIX_RELEASE(chain);
@@ -429,9 +435,7 @@ static void check_cached_events(pmix_rshift_caddy_t *cd)
                     }
                     chain->naffected = 1;
                     memcpy(chain->affected, ncd->info[n].value.data.proc, sizeof(pmix_proc_t));
-                } else if (0
-                           == strncmp(ncd->info[n].key, PMIX_EVENT_AFFECTED_PROCS,
-                                      PMIX_MAX_KEYLEN)) {
+                } else if (PMIX_CHECK_KEY(&ncd->info[n], PMIX_EVENT_AFFECTED_PROCS)) {
                     chain->naffected = ncd->info[n].value.data.darray->size;
                     PMIX_PROC_CREATE(chain->affected, chain->naffected);
                     if (NULL == chain->affected) {
