@@ -5,7 +5,7 @@
  * Copyright (c) 2015-2018 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * Copyright (c) 2018-2020 Intel, Inc.  All rights reserved.
- * Copyright (c) 2021-2022 Nanook Consulting.  All rights reserved.
+ * Copyright (c) 2021-2024 Nanook Consulting  All rights reserved.
  * Copyright (c) 2022      Triad National Security, LLC. All rights reserved.
  * $COPYRIGHT$
  *
@@ -39,29 +39,6 @@
 
 BEGIN_C_DECLS
 
-#if HWLOC_API_VERSION < 0x20000
-
-#    ifndef HAVE_HWLOC_TOPOLOGY_DUP
-#        define HAVE_HWLOC_TOPOLOGY_DUP 0
-#    endif
-
-#    define HWLOC_OBJ_L3CACHE HWLOC_OBJ_CACHE
-#    define HWLOC_OBJ_L2CACHE HWLOC_OBJ_CACHE
-#    define HWLOC_OBJ_L1CACHE HWLOC_OBJ_CACHE
-
-#    if HWLOC_API_VERSION < 0x10a00
-#        define HWLOC_OBJ_NUMANODE HWLOC_OBJ_NODE
-#        define HWLOC_OBJ_PACKAGE  HWLOC_OBJ_SOCKET
-#    endif
-
-#    define HAVE_DECL_HWLOC_OBJ_OSDEV_COPROC 0
-
-#else
-
-#    define HAVE_DECL_HWLOC_OBJ_OSDEV_COPROC 1
-
-#endif
-
 /**
  * Register params
  */
@@ -120,6 +97,8 @@ PMIX_EXPORT void pmix_hwloc_destruct_cpuset(pmix_cpuset_t *cpuset);
 
 PMIX_EXPORT void pmix_hwloc_release_cpuset(pmix_cpuset_t *ptr, size_t sz);
 
+PMIX_EXPORT pmix_status_t pmix_hwloc_get_cpuset_size(pmix_cpuset_t *ptr, size_t *sz);
+
 /* topology pack/unpack/copy/print functions */
 PMIX_EXPORT pmix_status_t pmix_hwloc_pack_topology(pmix_buffer_t *buf, pmix_topology_t *src,
                                                    pmix_pointer_array_t *regtypes);
@@ -135,6 +114,8 @@ PMIX_EXPORT void pmix_hwloc_destruct_topology(pmix_topology_t *ptr);
 
 PMIX_EXPORT void pmix_hwloc_release_topology(pmix_topology_t *ptr, size_t sz);
 
+PMIX_EXPORT pmix_status_t pmix_hwloc_get_topology_size(pmix_topology_t *ptr, size_t *sz);
+
 /****  PRESERVE ABI  ****/
 PMIX_EXPORT void pmix_ploc_base_destruct_cpuset(pmix_cpuset_t *cpuset);
 PMIX_EXPORT void pmix_ploc_base_release_cpuset(pmix_cpuset_t *ptr, size_t sz);
