@@ -95,6 +95,20 @@ typedef struct {
 } pmix_namelist_t;
 PMIX_CLASS_DECLARATION(pmix_namelist_t);
 
+/* define structs for holding entries in the
+ * show-help matrix */
+typedef struct {
+    const char *topic;
+    const char **content;
+} pmix_show_help_entry_t;
+
+typedef struct {
+    const char *filename;
+    pmix_show_help_entry_t *entries;
+} pmix_show_help_file_t;
+
+PMIX_EXPORT extern pmix_show_help_file_t pmix_show_help_data[];
+
 /* define a struct for holding entries in the
  * dictionary of attributes */
 typedef struct {
@@ -240,6 +254,9 @@ typedef uint8_t pmix_cmd_t;
 #define PMIX_FABRIC_UPDATE_CMD            31
 #define PMIX_COMPUTE_DEVICE_DISTANCES_CMD 32
 #define PMIX_REFRESH_CACHE                33
+#define PMIX_RESBLK_CMD                   34
+#define PMIX_SESSION_CTRL_CMD             35
+#define PMIX_REQ_SYSINFO_CMD              36
 #define PMIX_RESOLVE_PEERS_CMD            37
 #define PMIX_RESOLVE_NODE_CMD             38
 
@@ -349,6 +366,7 @@ typedef struct {
     size_t num_waiting;    // number of local procs waiting for debugger attach/release
     bool all_registered;   // all local ranks have been defined
     bool version_stored;   // the version string used by this nspace has been stored
+    bool job_info_recvd;   // job-level info has been received
     pmix_buffer_t *jobbkt; // packed version of jobinfo
     size_t ndelivered;     // count of #local clients that have received the jobinfo
     size_t nfinalized;     // count of #local clients that have finalized
@@ -453,6 +471,9 @@ typedef struct pmix_peer_t {
     int commit_cnt;
     pmix_epilog_t epilog; /**< things to be performed upon
                                termination of this peer */
+    uint32_t dyn_tags_start; // lower limit of valid tags for sendrecvs to this peer
+    uint32_t dyn_tags_current; // current tag for sendrecvs to this peer
+    uint32_t dyn_tags_end; // upper limit of valid tags for sendrecvs to this peer
 } pmix_peer_t;
 PMIX_CLASS_DECLARATION(pmix_peer_t);
 
@@ -504,6 +525,7 @@ PMIX_CLASS_DECLARATION(pmix_query_caddy_t);
 typedef struct {
     pmix_list_item_t super;
     char *grpid;
+    size_t ctxid;
     pmix_proc_t *members;
     size_t nmbrs;
 } pmix_group_t;
@@ -545,6 +567,7 @@ typedef struct {
     pmix_collect_t collect_type; // whether or not data is to be returned at completion
     pmix_modex_cbfunc_t modexcbfunc;
     pmix_op_cbfunc_t op_cbfunc;
+    pmix_info_cbfunc_t info_cbfunc;
     void *cbdata;
 } pmix_server_trkr_t;
 PMIX_CLASS_DECLARATION(pmix_server_trkr_t);
@@ -699,6 +722,7 @@ typedef struct {
     pmix_status_t status;
     pmix_proc_t source;
     pmix_data_range_t range;
+    bool staylocal;  // do not pass up to host environment
     /* For notification, we use the targets field to track
      * any custom range of procs that are to receive the
      * event.
