@@ -170,6 +170,10 @@ pmix_status_t pmix_bfrops_base_std_copy(void **dest, void *src, pmix_data_type_t
         datasize = sizeof(pmix_alloc_directive_t);
         break;
 
+    case PMIX_RESBLOCK_DIRECTIVE:
+        datasize = sizeof(pmix_resource_block_directive_t);
+        break;
+
     case PMIX_JOB_STATE:
         datasize = sizeof(pmix_job_state_t);
         break;
@@ -419,6 +423,47 @@ pmix_status_t pmix_bfrops_base_copy_geometry(pmix_geometry_t **dest, pmix_geomet
     return pmix_bfrops_base_tma_copy_geometry(dest, src, type, NULL);
 }
 
+pmix_status_t pmix_bfrops_base_copy_device(pmix_device_t **dest,
+                                           pmix_device_t *src, pmix_data_type_t type)
+{
+    pmix_device_t *dst;
+
+    PMIX_HIDE_UNUSED_PARAMS(type);
+
+    PMIX_DEVICE_CREATE(dst, 1);
+    if (NULL == dst) {
+        return PMIX_ERR_NOMEM;
+    }
+
+    if (NULL != src->uuid) {
+        dst->uuid = strdup(src->uuid);
+    }
+    if (NULL != src->osname) {
+        dst->osname = strdup(src->osname);
+    }
+    dst->type = src->type;
+
+    *dest = dst;
+    return PMIX_SUCCESS;
+}
+
+pmix_status_t pmix_bfrops_base_copy_resunit(pmix_resource_unit_t **dest,
+                                            pmix_resource_unit_t *src, pmix_data_type_t type)
+{
+    pmix_resource_unit_t *dst;
+
+    PMIX_HIDE_UNUSED_PARAMS(type);
+
+    PMIX_RESOURCE_UNIT_CREATE(dst, 1);
+    if (NULL == dst) {
+        return PMIX_ERR_NOMEM;
+    }
+    memcpy(dst, src, sizeof(pmix_resource_unit_t));
+
+    *dest = dst;
+    return PMIX_SUCCESS;
+}
+
 pmix_status_t pmix_bfrops_base_copy_devdist(pmix_device_distance_t **dest,
                                             pmix_device_distance_t *src, pmix_data_type_t type)
 {
