@@ -12,7 +12,7 @@
  * Copyright (c) 2015-2020 Intel, Inc.  All rights reserved.
  * Copyright (c) 2019      Mellanox Technologies, Inc.
  *                         All rights reserved.
- * Copyright (c) 2021-2022 Nanook Consulting.  All rights reserved.
+ * Copyright (c) 2021-2024 Nanook Consulting  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -119,6 +119,57 @@ pmix_status_t pmix_bfrops_base_pack_bool(pmix_pointer_array_t *regtypes, pmix_bu
     return PMIX_SUCCESS;
 }
 
+/*
+ * INT16, INT32, INT64, UIN16, UINT32, UINT64
+ */
+pmix_status_t pmix_bfrops_base_pack_general_int(pmix_pointer_array_t *regtypes,
+                                                pmix_buffer_t *buffer, const void *src,
+                                                int32_t num_vals, pmix_data_type_t type)
+{
+    pmix_status_t rc;
+    int32_t i;
+    char *dst;
+    size_t val_size, max_size, pkg_size;
+
+    pmix_output_verbose(20, pmix_bfrops_base_framework.framework_output,
+                        "pmix_bfrops_base_pack_integer * %d\n", num_vals);
+
+    PMIX_HIDE_UNUSED_PARAMS(regtypes);
+
+    PMIX_SQUASH_TYPE_SIZEOF(rc, type, val_size);
+    if (PMIX_SUCCESS != rc) {
+        PMIX_ERROR_LOG(rc);
+        return rc;
+    }
+
+    rc = pmix_bfrops_base_get_max_size(type, &max_size);
+    if (PMIX_SUCCESS != rc) {
+        PMIX_ERROR_LOG(rc);
+        return rc;
+    }
+
+    /* check to see if buffer needs extending */
+    if (NULL == (dst = pmix_bfrop_buffer_extend(buffer, num_vals * max_size))) {
+        rc = PMIX_ERR_OUT_OF_RESOURCE;
+        PMIX_ERROR_LOG(rc);
+        return rc;
+    }
+
+    for (i = 0; i < num_vals; ++i) {
+        rc = pmix_bfrops_base_encode_int(type, (uint8_t *) src + i * val_size, dst, &pkg_size);
+        if (PMIX_SUCCESS != rc) {
+            PMIX_ERROR_LOG(rc);
+            return rc;
+        }
+        dst += pkg_size;
+        buffer->pack_ptr += pkg_size;
+        buffer->bytes_used += pkg_size;
+    }
+
+    return PMIX_SUCCESS;
+}
+
+
 /*
  * INT
  */
@@ -129,12 +180,6 @@ pmix_status_t pmix_bfrops_base_pack_int(pmix_pointer_array_t *regtypes, pmix_buf
 
     PMIX_HIDE_UNUSED_PARAMS(type);
 
-    /* System types need to always be described so we can properly
-       unpack them */
-    if (PMIX_SUCCESS != (ret = pmix_bfrop_store_data_type(regtypes, buffer, BFROP_TYPE_INT))) {
-        return ret;
-    }
-
     /* Turn around and pack the real type */
     PMIX_BFROPS_PACK_TYPE(ret, buffer, src, num_vals, BFROP_TYPE_INT, regtypes);
     return ret;
@@ -150,12 +195,6 @@ pmix_status_t pmix_bfrops_base_pack_sizet(pmix_pointer_array_t *regtypes, pmix_b
 
     PMIX_HIDE_UNUSED_PARAMS(type);
 
-    /* System types need to always be described so we can properly
-       unpack them. */
-    if (PMIX_SUCCESS != (ret = pmix_bfrop_store_data_type(regtypes, buffer, BFROP_TYPE_SIZE_T))) {
-        return ret;
-    }
-
     PMIX_BFROPS_PACK_TYPE(ret, buffer, src, num_vals, BFROP_TYPE_SIZE_T, regtypes);
     return ret;
 }
@@ -209,99 +248,6 @@ pmix_status_t pmix_bfrops_base_pack_byte(pmix_pointer_array_t *regtypes, pmix_bu
     return PMIX_SUCCESS;
 }
 
-/*
- * INT16
- */
-pmix_status_t pmix_bfrops_base_pack_int16(pmix_pointer_array_t *regtypes, pmix_buffer_t *buffer,
-                                          const void *src, int32_t num_vals, pmix_data_type_t type)
-{
-    int32_t i;
-    uint16_t tmp, *srctmp = (uint16_t *) src;
-    char *dst;
-
-    pmix_output_verbose(20, pmix_bfrops_base_framework.framework_output,
-                        "pmix_bfrops_base_pack_int16 * %d\n", num_vals);
-
-    PMIX_HIDE_UNUSED_PARAMS(regtypes, type);
-
-    /* check to see if buffer needs extending */
-    if (NULL == (dst = pmix_bfrop_buffer_extend(buffer, num_vals * sizeof(tmp)))) {
-        return PMIX_ERR_OUT_OF_RESOURCE;
-    }
-
-    for (i = 0; i < num_vals; ++i) {
-        tmp = pmix_htons(srctmp[i]);
-        memcpy(dst, &tmp, sizeof(tmp));
-        dst += sizeof(tmp);
-    }
-    buffer->pack_ptr += num_vals * sizeof(tmp);
-    buffer->bytes_used += num_vals * sizeof(tmp);
-
-    return PMIX_SUCCESS;
-}
-
-/*
- * INT32
- */
-pmix_status_t pmix_bfrops_base_pack_int32(pmix_pointer_array_t *regtypes, pmix_buffer_t *buffer,
-                                          const void *src, int32_t num_vals, pmix_data_type_t type)
-{
-    int32_t i;
-    uint32_t tmp, *srctmp = (uint32_t *) src;
-    char *dst;
-
-    pmix_output_verbose(20, pmix_bfrops_base_framework.framework_output,
-                        "pmix_bfrops_base_pack_int32 * %d\n", num_vals);
-
-    PMIX_HIDE_UNUSED_PARAMS(regtypes, type);
-
-    /* check to see if buffer needs extending */
-    if (NULL == (dst = pmix_bfrop_buffer_extend(buffer, num_vals * sizeof(tmp)))) {
-        return PMIX_ERR_OUT_OF_RESOURCE;
-    }
-    for (i = 0; i < num_vals; ++i) {
-        tmp = htonl(srctmp[i]);
-        memcpy(dst, &tmp, sizeof(tmp));
-        dst += sizeof(tmp);
-    }
-    buffer->pack_ptr += num_vals * sizeof(tmp);
-    buffer->bytes_used += num_vals * sizeof(tmp);
-
-    return PMIX_SUCCESS;
-}
-
-/*
- * INT64
- */
-pmix_status_t pmix_bfrops_base_pack_int64(pmix_pointer_array_t *regtypes, pmix_buffer_t *buffer,
-                                          const void *src, int32_t num_vals, pmix_data_type_t type)
-{
-    int32_t i;
-    uint64_t tmp, tmp2;
-    char *dst;
-    size_t bytes_packed = num_vals * sizeof(tmp);
-
-    pmix_output_verbose(20, pmix_bfrops_base_framework.framework_output,
-                        "pmix_bfrops_base_pack_int64 * %d\n", num_vals);
-
-    PMIX_HIDE_UNUSED_PARAMS(regtypes, type);
-
-    /* check to see if buffer needs extending */
-    if (NULL == (dst = pmix_bfrop_buffer_extend(buffer, bytes_packed))) {
-        return PMIX_ERR_OUT_OF_RESOURCE;
-    }
-
-    for (i = 0; i < num_vals; ++i) {
-        memcpy(&tmp2, (char *) src + i * sizeof(uint64_t), sizeof(uint64_t));
-        tmp = pmix_hton64(tmp2);
-        memcpy(dst, &tmp, sizeof(tmp));
-        dst += sizeof(tmp);
-    }
-    buffer->pack_ptr += bytes_packed;
-    buffer->bytes_used += bytes_packed;
-
-    return PMIX_SUCCESS;
-}
 
 /*
  * STRING
@@ -661,7 +607,7 @@ pmix_status_t pmix_bfrops_base_pack_app(pmix_pointer_array_t *regtypes, pmix_buf
             return ret;
         }
         /* argv */
-        nvals = pmix_argv_count(app[i].argv);
+        nvals = PMIx_Argv_count(app[i].argv);
         /* although nvals is technically an int32, we have to pack it
          * as a generic int due to a typo in earlier release series. This
          * preserves the ordering of bytes in the packed buffer as it
@@ -680,7 +626,7 @@ pmix_status_t pmix_bfrops_base_pack_app(pmix_pointer_array_t *regtypes, pmix_buf
             }
         }
         /* env */
-        nvals = pmix_argv_count(app[i].env);
+        nvals = PMIx_Argv_count(app[i].env);
         PMIX_BFROPS_PACK_TYPE(ret, buffer, &nvals, 1, PMIX_INT32, regtypes);
         if (PMIX_SUCCESS != ret) {
             return ret;
@@ -930,7 +876,7 @@ pmix_status_t pmix_bfrops_base_pack_query(pmix_pointer_array_t *regtypes, pmix_b
 
     for (i = 0; i < num_vals; i++) {
         /* pack the number of keys */
-        nkeys = pmix_argv_count(pq[i].keys);
+        nkeys = PMIx_Argv_count(pq[i].keys);
         PMIX_BFROPS_PACK_TYPE(ret, buffer, &nkeys, 1, PMIX_INT32, regtypes);
         if (PMIX_SUCCESS != ret) {
             return ret;
@@ -978,6 +924,7 @@ pmix_status_t pmix_bfrops_base_pack_val(pmix_pointer_array_t *regtypes, pmix_buf
     case PMIX_TOPO:
     case PMIX_PROC_CPUSET:
     case PMIX_GEOMETRY:
+    case PMIX_DEVICE:
     case PMIX_DEVICE_DIST:
     case PMIX_ENDPOINT:
     case PMIX_REGATTR:
@@ -985,6 +932,7 @@ pmix_status_t pmix_bfrops_base_pack_val(pmix_pointer_array_t *regtypes, pmix_buf
     case PMIX_DISK_STATS:
     case PMIX_NET_STATS:
     case PMIX_NODE_STATS:
+    case PMIX_RESOURCE_UNIT:
         PMIX_BFROPS_PACK_TYPE(ret, buffer, p->data.ptr, 1, p->type, regtypes);
         if (PMIX_SUCCESS != ret) {
             return ret;
@@ -1016,6 +964,18 @@ pmix_status_t pmix_bfrops_base_pack_alloc_directive(pmix_pointer_array_t *regtyp
     return ret;
 }
 
+pmix_status_t pmix_bfrops_base_pack_resblock_directive(pmix_pointer_array_t *regtypes,
+                                                       pmix_buffer_t *buffer, const void *src,
+                                                       int32_t num_vals, pmix_data_type_t type)
+{
+    pmix_status_t ret;
+
+    PMIX_HIDE_UNUSED_PARAMS(type);
+
+    PMIX_BFROPS_PACK_TYPE(ret, buffer, src, num_vals, PMIX_UINT8, regtypes);
+    return ret;
+}
+
 pmix_status_t pmix_bfrops_base_pack_iof_channel(pmix_pointer_array_t *regtypes,
                                                 pmix_buffer_t *buffer, const void *src,
                                                 int32_t num_vals, pmix_data_type_t type)
@@ -1116,7 +1076,7 @@ pmix_status_t pmix_bfrops_base_pack_regattr(pmix_pointer_array_t *regtypes, pmix
             return ret;
         }
         /* pack the description */
-        nd = pmix_argv_count(ptr[i].description);
+        nd = PMIx_Argv_count(ptr[i].description);
         PMIX_BFROPS_PACK_TYPE(ret, buffer, &nd, 1, PMIX_INT32, regtypes);
         if (PMIX_SUCCESS != ret) {
             PMIX_ERROR_LOG(ret);
@@ -1227,6 +1187,56 @@ pmix_status_t pmix_bfrops_base_pack_geometry(pmix_pointer_array_t *regtypes, pmi
     return PMIX_SUCCESS;
 }
 
+pmix_status_t pmix_bfrops_base_pack_device(pmix_pointer_array_t *regtypes, pmix_buffer_t *buffer,
+                                          const void *src, int32_t num_vals,
+                                          pmix_data_type_t type)
+{
+    pmix_device_t *ptr = (pmix_device_t *) src;
+    int32_t i;
+    pmix_status_t ret;
+
+    PMIX_HIDE_UNUSED_PARAMS(type);
+
+    for (i = 0; i < num_vals; ++i) {
+        PMIX_BFROPS_PACK_TYPE(ret, buffer, &ptr[i].uuid, 1, PMIX_STRING, regtypes);
+        if (PMIX_SUCCESS != ret) {
+            return ret;
+        }
+        PMIX_BFROPS_PACK_TYPE(ret, buffer, &ptr[i].osname, 1, PMIX_STRING, regtypes);
+        if (PMIX_SUCCESS != ret) {
+            return ret;
+        }
+        PMIX_BFROPS_PACK_TYPE(ret, buffer, &ptr[i].type, 1, PMIX_DEVTYPE, regtypes);
+        if (PMIX_SUCCESS != ret) {
+            return ret;
+        }
+    }
+    return PMIX_SUCCESS;
+}
+
+pmix_status_t pmix_bfrops_base_pack_resunit(pmix_pointer_array_t *regtypes, pmix_buffer_t *buffer,
+                                            const void *src, int32_t num_vals,
+                                            pmix_data_type_t type)
+{
+    pmix_resource_unit_t *ptr = (pmix_resource_unit_t *) src;
+    int32_t i;
+    pmix_status_t ret;
+
+    PMIX_HIDE_UNUSED_PARAMS(type);
+
+    for (i = 0; i < num_vals; ++i) {
+        PMIX_BFROPS_PACK_TYPE(ret, buffer, &ptr[i].type, 1, PMIX_DEVTYPE, regtypes);
+        if (PMIX_SUCCESS != ret) {
+            return ret;
+        }
+        PMIX_BFROPS_PACK_TYPE(ret, buffer, &ptr[i].count, 1, PMIX_SIZE, regtypes);
+        if (PMIX_SUCCESS != ret) {
+            return ret;
+        }
+    }
+    return PMIX_SUCCESS;
+}
+
 pmix_status_t pmix_bfrops_base_pack_devdist(pmix_pointer_array_t *regtypes, pmix_buffer_t *buffer,
                                             const void *src, int32_t num_vals,
                                             pmix_data_type_t type)
