@@ -14,7 +14,7 @@
  * Copyright (c) 2016      Mellanox Technologies, Inc.
  *                         All rights reserved.
  * Copyright (c) 2022      Triad National Security, LLC. All rights reserved.
- * Copyright (c) 2021-2022 Nanook Consulting  All rights reserved.
+ * Copyright (c) 2021-2025 Nanook Consulting  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -44,14 +44,7 @@ char* PMIx_Info_string(const pmix_info_t *info)
     pmix_status_t rc;
     char *output = NULL;
 
-    if (pmix_globals.init_cntr <= 0) {
-        pmix_bfrops_base_print_info(&output, NULL, (void*)info, PMIX_INFO);
-        return output;
-    }
-
-    PMIX_BFROPS_PRINT(rc, pmix_globals.mypeer,
-                      &output, NULL,
-                      (void*)info, PMIX_INFO);
+    rc = pmix_bfrops_base_print_info(&output, NULL, (void*)info, PMIX_INFO);
     if (PMIX_SUCCESS != rc) {
         return NULL;
     }
@@ -63,14 +56,7 @@ char* PMIx_Value_string(const pmix_value_t *value)
     pmix_status_t rc;
     char *output = NULL;
 
-    if (pmix_globals.init_cntr <= 0) {
-        pmix_bfrops_base_print_value(&output, NULL, (void*)value, PMIX_VALUE);
-        return output;
-    }
-
-    PMIX_BFROPS_PRINT(rc, pmix_globals.mypeer,
-                      &output, NULL,
-                      (void*)value, PMIX_VALUE);
+    rc = pmix_bfrops_base_print_value(&output, NULL, (void*)value, PMIX_VALUE);
     if (PMIX_SUCCESS != rc) {
         return NULL;
     }
@@ -756,6 +742,9 @@ static int print_val(char **output, pmix_value_t *src)
         case PMIX_ALLOC_DIRECTIVE:
             rc = pmix_bfrops_base_print_alloc_directive(&tp, NULL, &src->data.adir, PMIX_ALLOC_DIRECTIVE);
             break;
+        case PMIX_RESBLOCK_DIRECTIVE:
+            rc = pmix_bfrops_base_print_resblock_directive(&tp, NULL, &src->data.adir, PMIX_RESBLOCK_DIRECTIVE);
+            break;
         case PMIX_ENVAR:
             rc = pmix_bfrops_base_print_envar(&tp, NULL, &src->data.envar, PMIX_ENVAR);
             break;
@@ -783,6 +772,9 @@ static int print_val(char **output, pmix_value_t *src)
         case PMIX_DEVTYPE:
             rc = pmix_bfrops_base_print_devtype(&tp, NULL, &src->data.devtype, PMIX_DEVTYPE);
             break;
+        case PMIX_DEVICE:
+            rc = pmix_bfrops_base_print_device(&tp, NULL, src->data.device, PMIX_DEVICE);
+            break;
         case PMIX_DEVICE_DIST:
             rc = pmix_bfrops_base_print_devdist(&tp, NULL, src->data.devdist, PMIX_DEVICE_DIST);
             break;
@@ -1025,7 +1017,7 @@ pmix_status_t pmix_bfrops_base_print_info_directives(char **output, char *prefix
     PMIX_HIDE_UNUSED_PARAMS(type);
 
     tmp = PMIx_Info_directives_string(*src);
-    if (0 > asprintf(output, "%sData type: PMIX_INFO_DIRECTIVES\tValue: %s",
+    if (0 > asprintf(output, "%sPMIX_INFO_DIRECTIVES\tValue: %s",
                      (NULL == prefix) ? " " : prefix, tmp)) {
         free(tmp);
         return PMIX_ERR_NOMEM;
@@ -1158,7 +1150,9 @@ pmix_status_t pmix_bfrops_base_print_pinfo(char **output, char *prefix,
                      "%sData type: PMIX_PROC_INFO\tValue:\n%s\n%sHostname: %s\tExecutable: "
                      "%s\n%sPid: %lu\tExit code: %d\tState: %s",
                      (NULL == prefix) ? " " : prefix, tmp, p2,
-                     src->hostname, src->executable_name, p2,
+                     (NULL == src->hostname) ? "NULL" : src->hostname,
+                     (NULL == src->executable_name) ? "NULL" : src->executable_name,
+                     p2,
                      (unsigned long) src->pid, src->exit_code,
                      PMIx_Proc_state_string(src->state))) {
         free(p2);
@@ -1209,6 +1203,7 @@ pmix_status_t pmix_bfrops_base_print_darray(char **output, char *prefix,
     pmix_data_array_t *daptr;
     pmix_regattr_t *rgptr;
     pmix_alloc_directive_t *adptr;
+    pmix_resource_block_directive_t *rbptr;
     pmix_envar_t *evptr;
     pmix_coord_t *coptr;
     pmix_link_state_t *lkptr;
@@ -1218,6 +1213,8 @@ pmix_status_t pmix_bfrops_base_print_darray(char **output, char *prefix,
     pmix_locality_t *lcptr;
     pmix_geometry_t *geoptr;
     pmix_device_type_t *dvptr;
+    pmix_device_t *dev;
+    pmix_resource_unit_t *resunit;
     pmix_device_distance_t *ddptr;
     pmix_endpoint_t *endptr;
     pmix_storage_medium_t *smptr;
@@ -1362,6 +1359,10 @@ pmix_status_t pmix_bfrops_base_print_darray(char **output, char *prefix,
                 adptr = (pmix_alloc_directive_t*)src->array;
                 rc = pmix_bfrops_base_print_alloc_directive(&tp, prefix, &adptr[n], PMIX_ALLOC_DIRECTIVE);
                 break;
+            case PMIX_RESBLOCK_DIRECTIVE:
+                rbptr = (pmix_resource_block_directive_t*)src->array;
+                rc = pmix_bfrops_base_print_resblock_directive(&tp, prefix, &rbptr[n], PMIX_RESBLOCK_DIRECTIVE);
+                break;
             case PMIX_ENVAR:
                 evptr = (pmix_envar_t*)src->array;
                 rc = pmix_bfrops_base_print_envar(&tp, prefix, &evptr[n], PMIX_ENVAR);
@@ -1398,6 +1399,14 @@ pmix_status_t pmix_bfrops_base_print_darray(char **output, char *prefix,
                 dvptr = (pmix_device_type_t*)src->array;
                 rc = pmix_bfrops_base_print_devtype(&tp, prefix, &dvptr[n], PMIX_DEVTYPE);
                 break;
+            case PMIX_DEVICE:
+                dev = (pmix_device_t*)src->array;
+                rc = pmix_bfrops_base_print_device(&tp, prefix, &dev[n], PMIX_DEVICE);
+                break;
+            case PMIX_RESOURCE_UNIT:
+                resunit = (pmix_resource_unit_t*)src->array;
+                rc = pmix_bfrops_base_print_resunit(&tp, prefix, &resunit[n], PMIX_RESOURCE_UNIT);
+                break;
             case PMIX_DEVICE_DIST:
                 ddptr = (pmix_device_distance_t*)src->array;
                 rc = pmix_bfrops_base_print_devdist(&tp, prefix, &ddptr[n], PMIX_DEVICE_DIST);
@@ -1563,6 +1572,25 @@ pmix_status_t pmix_bfrops_base_print_alloc_directive(char **output, char *prefix
     }
 }
 
+pmix_status_t pmix_bfrops_base_print_resblock_directive(char **output, char *prefix,
+                                                        pmix_resource_block_directive_t *src,
+                                                        pmix_data_type_t type)
+{
+    int ret;
+
+    PMIX_HIDE_UNUSED_PARAMS(type);
+
+    ret = asprintf(output, "%sData type: PMIX_RESBLOCK_DIRECTIVE\tValue: %s",
+                   (NULL == prefix) ? " " : prefix,
+                   PMIx_Resource_block_directive_string(*src));
+
+    if (0 > ret) {
+        return PMIX_ERR_OUT_OF_RESOURCE;
+    } else {
+        return PMIX_SUCCESS;
+    }
+}
+
 pmix_status_t pmix_bfrops_base_print_iof_channel(char **output, char *prefix,
                                                  pmix_iof_channel_t *src, pmix_data_type_t type)
 {
@@ -1764,6 +1792,45 @@ pmix_status_t pmix_bfrops_base_print_geometry(char **output, char *prefix, pmix_
     return PMIX_SUCCESS;
 }
 
+pmix_status_t pmix_bfrops_base_print_device(char **output, char *prefix,
+                                            pmix_device_t *src, pmix_data_type_t type)
+{
+    int ret;
+
+    PMIX_HIDE_UNUSED_PARAMS(type);
+
+    ret = asprintf(output,
+                   "%sData type: PMIX_DEVICE\tValue: UUID: %s OSName: %s Type: %s",
+                   (NULL == prefix) ? " " : prefix,
+                   (NULL == src->uuid) ? "NULL" : src->uuid, (NULL == src->osname) ? "NULL" : src->osname,
+                   PMIx_Device_type_string(src->type));
+
+    if (0 > ret) {
+        return PMIX_ERR_OUT_OF_RESOURCE;
+    } else {
+        return PMIX_SUCCESS;
+    }
+}
+
+pmix_status_t pmix_bfrops_base_print_resunit(char **output, char *prefix,
+                                             pmix_resource_unit_t *src, pmix_data_type_t type)
+{
+    int ret;
+
+    PMIX_HIDE_UNUSED_PARAMS(type);
+
+    ret = asprintf(output,
+                   "%sData type: PMIX_RESOURCE_UNIT\tValue: Type: %s  Count: %" PRIsize_t "",
+                   (NULL == prefix) ? " " : prefix,
+                   PMIx_Device_type_string(src->type), src->count);
+
+    if (0 > ret) {
+        return PMIX_ERR_OUT_OF_RESOURCE;
+    } else {
+        return PMIX_SUCCESS;
+    }
+}
+
 pmix_status_t pmix_bfrops_base_print_devdist(char **output, char *prefix,
                                              pmix_device_distance_t *src, pmix_data_type_t type)
 {
