@@ -15,7 +15,7 @@
  *                         and Technology (RIST).  All rights reserved.
  * Copyright (c) 2016-2019 Mellanox Technologies, Inc.
  *                         All rights reserved.
- * Copyright (c) 2021-2022 Nanook Consulting.  All rights reserved.
+ * Copyright (c) 2021-2024 Nanook Consulting  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -54,9 +54,9 @@ static pmix_status_t pmix_bfrops_base_unpack_buffer(pmix_pointer_array_t *regtyp
         }
         /* if the data types don't match, then return an error */
         if (type != local_type) {
-            pmix_output(0, "PMIX bfrop:unpack: got type %s when expecting type %s",
-                        PMIx_Data_type_string(local_type),
-                        PMIx_Data_type_string(type));
+            pmix_output(0, "PMIx bfrop:unpack: got type %s when expecting type %s",
+                           PMIx_Data_type_string(local_type),
+                           PMIx_Data_type_string(type));
             return PMIX_ERR_PACK_MISMATCH;
         }
     }
@@ -73,6 +73,10 @@ pmix_status_t pmix_bfrops_base_unpack(pmix_pointer_array_t *regtypes, pmix_buffe
 
     /* check for error */
     if (NULL == buffer || NULL == dst || NULL == num_vals) {
+        pmix_output(0, "SOMEONE IS NULL: buffer %s dst %s num_vals %s",
+            (NULL == buffer) ? "NULL" : "GOOD",
+            (NULL == dst) ? "NULL" : "GOOD",
+            (NULL == num_vals) ? "NULL" : "GOOD");
         return PMIX_ERR_BAD_PARAM;
     }
 
@@ -137,8 +141,8 @@ pmix_status_t pmix_bfrops_base_unpack(pmix_pointer_array_t *regtypes, pmix_buffe
     }
 
     /** Unpack the value(s) */
-    if (PMIX_SUCCESS
-        != (rc = pmix_bfrops_base_unpack_buffer(regtypes, buffer, dst, &local_num, type))) {
+    rc = pmix_bfrops_base_unpack_buffer(regtypes, buffer, dst, &local_num, type);
+    if (PMIX_SUCCESS != rc) {
         *num_vals = 0;
         ret = rc;
     }
@@ -186,6 +190,62 @@ pmix_status_t pmix_bfrops_base_unpack_bool(pmix_pointer_array_t *regtypes, pmix_
     return PMIX_SUCCESS;
 }
 
+pmix_status_t pmix_bfrops_base_unpack_general_int(pmix_pointer_array_t *regtypes,
+                                                  pmix_buffer_t *buffer, void *dest,
+                                                  int32_t *num_vals, pmix_data_type_t type)
+{
+    pmix_status_t rc;
+    size_t val_size, avail_size, unpack_size, max_size;
+    int32_t i;
+
+    pmix_output_verbose(20, pmix_bfrops_base_framework.framework_output,
+                        "pmix_bfrops_base_unpack_integer * %d\n", (int) *num_vals);
+
+    PMIX_HIDE_UNUSED_PARAMS(regtypes, type);
+
+    /* check to see if there's enough data in buffer */
+    if (buffer->pack_ptr == buffer->unpack_ptr) {
+        return PMIX_ERR_UNPACK_READ_PAST_END_OF_BUFFER;
+    }
+
+    PMIX_SQUASH_TYPE_SIZEOF(rc, type, val_size);
+    if (PMIX_SUCCESS != rc) {
+        PMIX_ERROR_LOG(rc);
+        return rc;
+    }
+
+    rc = pmix_bfrops_base_get_max_size(type, &max_size);
+    if (PMIX_SUCCESS != rc) {
+        PMIX_ERROR_LOG(rc);
+        return rc;
+    }
+
+    /* unpack the data */
+    for (i = 0; i < (*num_vals); ++i) {
+        avail_size = buffer->pack_ptr - buffer->unpack_ptr;
+        rc = pmix_bfrops_base_decode_int(type, buffer->unpack_ptr, avail_size,
+                                         (uint8_t *) dest + i * val_size, &unpack_size);
+        if (PMIX_SUCCESS != rc) {
+            PMIX_ERROR_LOG(rc);
+            return rc;
+        }
+        /* sanity checks */
+        if (unpack_size > max_size) {
+            rc = PMIX_ERR_UNPACK_FAILURE;
+            PMIX_ERROR_LOG(rc);
+            return rc;
+        }
+        if (unpack_size > avail_size) {
+            rc = PMIX_ERR_FATAL;
+            PMIX_ERROR_LOG(rc);
+            return rc;
+        }
+        buffer->unpack_ptr += unpack_size;
+    }
+
+    return PMIX_SUCCESS;
+}
+
 /*
  * INT
  */
@@ -193,22 +253,9 @@ pmix_status_t pmix_bfrops_base_unpack_int(pmix_pointer_array_t *regtypes, pmix_b
                                           void *dest, int32_t *num_vals, pmix_data_type_t type)
 {
     pmix_status_t ret;
-    pmix_data_type_t remote_type;
-
     PMIX_HIDE_UNUSED_PARAMS(type);
 
-    if (PMIX_SUCCESS != (ret = pmix_bfrop_get_data_type(regtypes, buffer, &remote_type))) {
-        return ret;
-    }
-
-    if (remote_type == BFROP_TYPE_INT) {
-        /* fast path it if the sizes are the same */
-        /* Turn around and unpack the real type */
-        PMIX_BFROPS_UNPACK_TYPE(ret, buffer, dest, num_vals, BFROP_TYPE_INT, regtypes);
-    } else {
-        /* slow path - types are different sizes */
-        PMIX_BFROP_UNPACK_SIZE_MISMATCH(regtypes, int, remote_type, ret);
-    }
+    PMIX_BFROPS_UNPACK_TYPE(ret, buffer, dest, num_vals, BFROP_TYPE_INT, regtypes);
 
     return ret;
 }
@@ -220,29 +267,11 @@ pmix_status_t pmix_bfrops_base_unpack_sizet(pmix_pointer_array_t *regtypes, pmix
                                             void *dest, int32_t *num_vals, pmix_data_type_t type)
 {
     pmix_status_t ret;
-    pmix_data_type_t remote_type;
-
-    if (PMIX_SIZE != type) {
-        return PMIX_ERR_BAD_PARAM;
-    }
-
     PMIX_HIDE_UNUSED_PARAMS(type);
 
-    if (PMIX_SUCCESS != (ret = pmix_bfrop_get_data_type(regtypes, buffer, &remote_type))) {
+    PMIX_BFROPS_UNPACK_TYPE(ret, buffer, dest, num_vals, BFROP_TYPE_SIZE_T, regtypes);
+    if (PMIX_SUCCESS != ret) {
         PMIX_ERROR_LOG(ret);
-        return ret;
-    }
-
-    if (remote_type == BFROP_TYPE_SIZE_T) {
-        /* fast path it if the sizes are the same */
-        /* Turn around and unpack the real type */
-        PMIX_BFROPS_UNPACK_TYPE(ret, buffer, dest, num_vals, BFROP_TYPE_SIZE_T, regtypes);
-        if (PMIX_SUCCESS != ret) {
-            PMIX_ERROR_LOG(ret);
-        }
-    } else {
-        /* slow path - types are different sizes */
-        PMIX_BFROP_UNPACK_SIZE_MISMATCH(regtypes, size_t, remote_type, ret);
     }
     return ret;
 }
@@ -301,60 +330,6 @@ pmix_status_t pmix_bfrops_base_unpack_byte(pmix_pointer_array_t *regtypes, pmix_
     return PMIX_SUCCESS;
 }
 
-pmix_status_t pmix_bfrops_base_unpack_int16(pmix_pointer_array_t *regtypes, pmix_buffer_t *buffer,
-                                            void *dest, int32_t *num_vals, pmix_data_type_t type)
-{
-    int32_t i;
-    uint16_t tmp, *desttmp = (uint16_t *) dest;
-
-    pmix_output_verbose(20, pmix_bfrops_base_framework.framework_output,
-                        "pmix_bfrop_unpack_int16 * %d\n", (int) *num_vals);
-
-    PMIX_HIDE_UNUSED_PARAMS(regtypes, type);
-
-    /* check to see if there's enough data in buffer */
-    if (pmix_bfrop_too_small(buffer, (*num_vals) * sizeof(tmp))) {
-        return PMIX_ERR_UNPACK_READ_PAST_END_OF_BUFFER;
-    }
-
-    /* unpack the data */
-    for (i = 0; i < (*num_vals); ++i) {
-        memcpy(&(tmp), buffer->unpack_ptr, sizeof(tmp));
-        tmp = pmix_ntohs(tmp);
-        memcpy(&desttmp[i], &tmp, sizeof(tmp));
-        buffer->unpack_ptr += sizeof(tmp);
-    }
-
-    return PMIX_SUCCESS;
-}
-
-pmix_status_t pmix_bfrops_base_unpack_int32(pmix_pointer_array_t *regtypes, pmix_buffer_t *buffer,
-                                            void *dest, int32_t *num_vals, pmix_data_type_t type)
-{
-    int32_t i;
-    uint32_t tmp, *desttmp = (uint32_t *) dest;
-
-    pmix_output_verbose(20, pmix_bfrops_base_framework.framework_output,
-                        "pmix_bfrop_unpack_int32 * %d\n", (int) *num_vals);
-
-    PMIX_HIDE_UNUSED_PARAMS(regtypes, type);
-
-    /* check to see if there's enough data in buffer */
-    if (pmix_bfrop_too_small(buffer, (*num_vals) * sizeof(tmp))) {
-        return PMIX_ERR_UNPACK_READ_PAST_END_OF_BUFFER;
-    }
-
-    /* unpack the data */
-    for (i = 0; i < (*num_vals); ++i) {
-        memcpy(&(tmp), buffer->unpack_ptr, sizeof(tmp));
-        tmp = ntohl(tmp);
-        memcpy(&desttmp[i], &tmp, sizeof(tmp));
-        buffer->unpack_ptr += sizeof(tmp);
-    }
-
-    return PMIX_SUCCESS;
-}
-
 pmix_status_t pmix_bfrops_base_unpack_datatype(pmix_pointer_array_t *regtypes,
                                                pmix_buffer_t *buffer, void *dest, int32_t *num_vals,
                                                pmix_data_type_t type)
@@ -367,33 +342,6 @@ pmix_status_t pmix_bfrops_base_unpack_datatype(pmix_pointer_array_t *regtypes,
     return ret;
 }
 
-pmix_status_t pmix_bfrops_base_unpack_int64(pmix_pointer_array_t *regtypes, pmix_buffer_t *buffer,
-                                            void *dest, int32_t *num_vals, pmix_data_type_t type)
-{
-    int32_t i;
-    uint64_t tmp, *desttmp = (uint64_t *) dest;
-
-    pmix_output_verbose(20, pmix_bfrops_base_framework.framework_output,
-                        "pmix_bfrop_unpack_int64 * %d\n", (int) *num_vals);
-
-    PMIX_HIDE_UNUSED_PARAMS(regtypes, type);
-
-    /* check to see if there's enough data in buffer */
-    if (pmix_bfrop_too_small(buffer, (*num_vals) * sizeof(tmp))) {
-        return PMIX_ERR_UNPACK_READ_PAST_END_OF_BUFFER;
-    }
-
-    /* unpack the data */
-    for (i = 0; i < (*num_vals); ++i) {
-        memcpy(&(tmp), buffer->unpack_ptr, sizeof(tmp));
-        tmp = pmix_ntoh64(tmp);
-        memcpy(&desttmp[i], &tmp, sizeof(tmp));
-        buffer->unpack_ptr += sizeof(tmp);
-    }
-
-    return PMIX_SUCCESS;
-}
-
 pmix_status_t pmix_bfrops_base_unpack_string(pmix_pointer_array_t *regtypes, pmix_buffer_t *buffer,
                                              void *dest, int32_t *num_vals, pmix_data_type_t type)
 {
@@ -628,6 +576,20 @@ pmix_status_t pmix_bfrops_base_unpack_val(pmix_pointer_array_t *regtypes, pmix_b
             }
             PMIX_BFROPS_UNPACK_TYPE(ret, buffer, val->data.geometry, &m, PMIX_GEOMETRY, regtypes);
             return ret;
+        case PMIX_DEVICE:
+            PMIX_DEVICE_CREATE(val->data.device, 1);
+            if (NULL == val->data.device) {
+                return PMIX_ERR_NOMEM;
+            }
+            PMIX_BFROPS_UNPACK_TYPE(ret, buffer, val->data.device, &m, PMIX_DEVICE, regtypes);
+            return ret;
+        case PMIX_RESOURCE_UNIT:
+            PMIX_RESOURCE_UNIT_CREATE(val->data.resunit, 1);
+            if (NULL == val->data.resunit) {
+                return PMIX_ERR_NOMEM;
+            }
+            PMIX_BFROPS_UNPACK_TYPE(ret, buffer, val->data.resunit, &m, PMIX_RESOURCE_UNIT, regtypes);
+            return ret;
         case PMIX_DEVICE_DIST:
             PMIX_DEVICE_DIST_CREATE(val->data.devdist, 1);
             if (NULL == val->data.devdist) {
@@ -975,7 +937,7 @@ pmix_status_t pmix_bfrops_base_unpack_app(pmix_pointer_array_t *regtypes, pmix_b
             if (NULL == tmp) {
                 return PMIX_ERROR;
             }
-            pmix_argv_append_nosize(&ptr[i].argv, tmp);
+            PMIx_Argv_append_nosize(&ptr[i].argv, tmp);
             free(tmp);
         }
         /* unpack env */
@@ -994,7 +956,7 @@ pmix_status_t pmix_bfrops_base_unpack_app(pmix_pointer_array_t *regtypes, pmix_b
             if (NULL == tmp) {
                 return PMIX_ERROR;
             }
-            pmix_argv_append_nosize(&ptr[i].env, tmp);
+            PMIx_Argv_append_nosize(&ptr[i].env, tmp);
             free(tmp);
         }
         /* unpack cwd */
@@ -1098,6 +1060,9 @@ pmix_status_t pmix_bfrops_base_unpack_bo(pmix_pointer_array_t *regtypes, pmix_bu
         }
         if (0 < ptr[i].size) {
             ptr[i].bytes = (char *) malloc(ptr[i].size * sizeof(char));
+            if (NULL == ptr[i].bytes) {
+                return PMIX_ERR_NOMEM;
+            }
             m = ptr[i].size;
             PMIX_BFROPS_UNPACK_TYPE(ret, buffer, ptr[i].bytes, &m, PMIX_BYTE, regtypes);
             if (PMIX_SUCCESS != ret) {
@@ -1359,6 +1324,18 @@ pmix_status_t pmix_bfrops_base_unpack_alloc_directive(pmix_pointer_array_t *regt
     return ret;
 }
 
+pmix_status_t pmix_bfrops_base_unpack_resblock_directive(pmix_pointer_array_t *regtypes,
+                                                         pmix_buffer_t *buffer, void *dest,
+                                                         int32_t *num_vals, pmix_data_type_t type)
+{
+    pmix_status_t ret;
+
+    PMIX_HIDE_UNUSED_PARAMS(type);
+
+    PMIX_BFROPS_UNPACK_TYPE(ret, buffer, dest, num_vals, PMIX_UINT8, regtypes);
+    return ret;
+}
+
 pmix_status_t pmix_bfrops_base_unpack_iof_channel(pmix_pointer_array_t *regtypes,
                                                   pmix_buffer_t *buffer, void *dest,
                                                   int32_t *num_vals, pmix_data_type_t type)
@@ -1653,6 +1630,81 @@ pmix_status_t pmix_bfrops_base_unpack_geometry(pmix_pointer_array_t *regtypes,
     return PMIX_SUCCESS;
 }
 
+pmix_status_t pmix_bfrops_base_unpack_device(pmix_pointer_array_t *regtypes, pmix_buffer_t *buffer,
+                                             void *dest, int32_t *num_vals, pmix_data_type_t type)
+{
+    pmix_device_t *ptr;
+    int32_t i, n, m;
+    pmix_status_t ret;
+
+    pmix_output_verbose(20, pmix_bfrops_base_framework.framework_output,
+                        "pmix_bfrop_unpack: %d devices", *num_vals);
+
+    PMIX_HIDE_UNUSED_PARAMS(type);
+
+    ptr = (pmix_device_t *) dest;
+    n = *num_vals;
+
+    for (i = 0; i < n; ++i) {
+        PMIX_DEVICE_CONSTRUCT(&ptr[i]);
+        /* unpack the uuid */
+        m = 1;
+        PMIX_BFROPS_UNPACK_TYPE(ret, buffer, &ptr[i].uuid, &m, PMIX_STRING, regtypes);
+        if (PMIX_SUCCESS != ret) {
+            PMIX_ERROR_LOG(ret);
+            return ret;
+        }
+        m = 1;
+        PMIX_BFROPS_UNPACK_TYPE(ret, buffer, &ptr[i].osname, &m, PMIX_STRING, regtypes);
+        if (PMIX_SUCCESS != ret) {
+            PMIX_ERROR_LOG(ret);
+            return ret;
+        }
+        m = 1;
+        PMIX_BFROPS_UNPACK_TYPE(ret, buffer, &ptr[i].type, &m, PMIX_DEVTYPE, regtypes);
+        if (PMIX_SUCCESS != ret) {
+            PMIX_ERROR_LOG(ret);
+            return ret;
+        }
+    }
+    return PMIX_SUCCESS;
+}
+
+pmix_status_t pmix_bfrops_base_unpack_resunit(pmix_pointer_array_t *regtypes, pmix_buffer_t *buffer,
+                                              void *dest, int32_t *num_vals, pmix_data_type_t type)
+{
+    pmix_resource_unit_t *ptr;
+    int32_t i, n, m;
+    pmix_status_t ret;
+
+    pmix_output_verbose(20, pmix_bfrops_base_framework.framework_output,
+                        "pmix_bfrop_unpack: %d resource units", *num_vals);
+
+    PMIX_HIDE_UNUSED_PARAMS(type);
+
+    ptr = (pmix_resource_unit_t *) dest;
+    n = *num_vals;
+
+    for (i = 0; i < n; ++i) {
+        PMIX_RESOURCE_UNIT_CONSTRUCT(&ptr[i]);
+        /* unpack the type */
+        m = 1;
+        PMIX_BFROPS_UNPACK_TYPE(ret, buffer, &ptr[i].type, &m, PMIX_DEVTYPE, regtypes);
+        if (PMIX_SUCCESS != ret) {
+            PMIX_ERROR_LOG(ret);
+            return ret;
+        }
+        /* get the number of them */
+        m = 1;
+        PMIX_BFROPS_UNPACK_TYPE(ret, buffer, &ptr[i].count, &m, PMIX_SIZE, regtypes);
+        if (PMIX_SUCCESS != ret) {
+            PMIX_ERROR_LOG(ret);
+            return ret;
+        }
+    }
+    return PMIX_SUCCESS;
+}
+
 pmix_status_t pmix_bfrops_base_unpack_devdist(pmix_pointer_array_t *regtypes, pmix_buffer_t *buffer,
                                               void *dest, int32_t *num_vals, pmix_data_type_t type)
 {
