@@ -6,7 +6,7 @@
  * Copyright (c) 2018-2020 Mellanox Technologies, Inc.
  *                         All rights reserved.
  * Copyright (c) 2022-2023 Triad National Security, LLC. All rights reserved.
- * Copyright (c) 2021-2024 Nanook Consulting  All rights reserved.
+ * Copyright (c) 2021-2025 Nanook Consulting  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -63,11 +63,10 @@ static pmix_status_t hash_register_job_info(struct pmix_peer_t *pr, pmix_buffer_
 
 static pmix_status_t hash_store_job_info(const char *nspace, pmix_buffer_t *buf);
 
-static pmix_status_t hash_store_modex(struct pmix_namespace_t *ns, pmix_buffer_t *buff,
+static pmix_status_t hash_store_modex(pmix_buffer_t *buff,
                                       void *cbdata);
 
-static pmix_status_t _hash_store_modex(pmix_gds_base_ctx_t ctx, pmix_proc_t *proc,
-                                       pmix_gds_modex_key_fmt_t key_fmt, char **kmap,
+static pmix_status_t _hash_store_modex(pmix_proc_t *proc,
                                        pmix_buffer_t *pbkt);
 
 static pmix_status_t setup_fork(const pmix_proc_t *peer, char ***env);
@@ -558,7 +557,7 @@ static pmix_status_t register_info(pmix_peer_t *peer,
 
     /* get any session-level info for this job */
     PMIX_CONSTRUCT(&results, pmix_list_t);
-    rc = pmix_gds_hash_fetch_sessioninfo(NULL, trk, NULL, 0, &results);
+    rc = pmix_gds_hash_fetch_sessioninfo(peer, NULL, trk, NULL, 0, &results);
     if (PMIX_SUCCESS == rc) {
         PMIX_LIST_FOREACH (kvptr, &results, pmix_kval_t) {
             PMIX_BFROPS_PACK(rc, peer, reply, kvptr, 1, PMIX_KVAL);
@@ -572,7 +571,7 @@ static pmix_status_t register_info(pmix_peer_t *peer,
         sptr = pmix_gds_hash_check_session(NULL, UINT32_MAX, false);
         if (NULL != sptr) {
             PMIX_CONSTRUCT(&results, pmix_list_t);
-            rc = pmix_gds_hash_xfer_sessioninfo(sptr, trk, NULL, &results);
+            rc = pmix_gds_hash_xfer_sessioninfo(peer, sptr, NULL, &results);
             if (PMIX_SUCCESS == rc) {
                 PMIX_LIST_FOREACH (kvptr, &results, pmix_kval_t) {
                     PMIX_BFROPS_PACK(rc, peer, reply, kvptr, 1, PMIX_KVAL);
@@ -584,7 +583,7 @@ static pmix_status_t register_info(pmix_peer_t *peer,
 
     /* get any node-level info for this job */
     PMIX_CONSTRUCT(&results, pmix_list_t);
-    rc = pmix_gds_hash_fetch_nodeinfo(NULL, trk, &trk->nodeinfo, NULL, 0, &results);
+    rc = pmix_gds_hash_fetch_nodeinfo(peer, NULL, &trk->nodeinfo, NULL, 0, &results);
     if (PMIX_SUCCESS == rc) {
         PMIX_LIST_FOREACH (kvptr, &results, pmix_kval_t) {
             /* if the peer is earlier than v3.2.x, it is expecting
@@ -612,6 +611,7 @@ static pmix_status_t register_info(pmix_peer_t *peer,
                             kv.key = strdup(info[n].key);
                             kv.value = &info[n].value;
                             PMIX_BFROPS_PACK(rc, peer, reply, &kv, 1, PMIX_KVAL);
+                            PMIX_DESTRUCT(&kv);
                         }
                     }
                 }
@@ -624,7 +624,7 @@ static pmix_status_t register_info(pmix_peer_t *peer,
 
     /* get any app-level info for this job */
     PMIX_CONSTRUCT(&results, pmix_list_t);
-    rc = pmix_gds_hash_fetch_appinfo(NULL, trk, &trk->apps, NULL, 0, &results);
+    rc = pmix_gds_hash_fetch_appinfo(peer, NULL, &trk->apps, NULL, 0, &results);
     if (PMIX_SUCCESS == rc) {
         PMIX_LIST_FOREACH (kvptr, &results, pmix_kval_t) {
             PMIX_BFROPS_PACK(rc, peer, reply, kvptr, 1, PMIX_KVAL);
@@ -1167,7 +1167,8 @@ pmix_status_t pmix_gds_hash_store(const pmix_proc_t *proc,
 
     pmix_output_verbose(2, pmix_gds_base_framework.framework_output,
                         "%s gds:hash:hash_store for proc %s key %s type %s scope %s",
-                        PMIX_NAME_PRINT(&pmix_globals.myid), PMIX_NAME_PRINT(proc), kv->key,
+                        PMIX_NAME_PRINT(&pmix_globals.myid), PMIX_NAME_PRINT(proc),
+                        PMIx_Get_attribute_name(kv->key),
                         PMIx_Data_type_string(kv->value->type), PMIx_Scope_string(scope));
 
     if (NULL == kv->key) {
@@ -1323,27 +1324,29 @@ pmix_status_t pmix_gds_hash_store(const pmix_proc_t *proc,
  * host has received data from some other peer. It therefore
  * always contains data solely from remote procs, and we
  * shall store it accordingly */
-static pmix_status_t hash_store_modex(struct pmix_namespace_t *nspace, pmix_buffer_t *buf,
+static pmix_status_t hash_store_modex(pmix_buffer_t *buf,
                                       void *cbdata)
 {
-    return pmix_gds_base_store_modex(nspace, buf, NULL, _hash_store_modex, cbdata);
+    return pmix_gds_base_store_modex(buf, _hash_store_modex, cbdata);
 }
 
-static pmix_status_t _hash_store_modex(pmix_gds_base_ctx_t ctx, pmix_proc_t *proc,
-                                       pmix_gds_modex_key_fmt_t key_fmt, char **kmap,
+static pmix_status_t _hash_store_modex(pmix_proc_t *proc,
                                        pmix_buffer_t *pbkt)
 {
     pmix_job_t *trk;
     pmix_status_t rc = PMIX_SUCCESS;
     pmix_kval_t kv;
+    int32_t cnt;
+
+    if (NULL == pbkt) {
+        return PMIX_SUCCESS;
+    }
 
     pmix_output_verbose(2, pmix_gds_base_framework.framework_output,
                         "[%s:%d] gds:hash:store_modex for nspace %s",
                         pmix_globals.myid.nspace,
                         pmix_globals.myid.rank, proc->nspace);
 
-    PMIX_HIDE_UNUSED_PARAMS(ctx);
-
     /* find the hash table for this nspace */
     trk = pmix_gds_hash_get_tracker(proc->nspace, true);
     if (NULL == trk) {
@@ -1353,12 +1356,13 @@ static pmix_status_t _hash_store_modex(pmix_gds_base_ctx_t ctx, pmix_proc_t *pro
     /* this is data returned via the PMIx_Fence call when
      * data collection was requested, so it only contains
      * REMOTE/GLOBAL data. The byte object contains
-     * the rank followed by pmix_kval_t's. The list of callbacks
-     * contains all local participants. */
+     * the pmix_kval_t's published by the given proc
+     */
 
-    /* unpack the remaining values until we hit the end of the buffer */
+    /* unpack the values until we hit the end of the buffer */
     PMIX_CONSTRUCT(&kv, pmix_kval_t);
-    rc = pmix_gds_base_modex_unpack_kval(key_fmt, pbkt, kmap, &kv);
+    cnt = 1;
+    PMIX_BFROPS_UNPACK(rc, pmix_globals.mypeer, pbkt, &kv, &cnt, PMIX_KVAL);
 
     while (PMIX_SUCCESS == rc) {
         if (PMIX_RANK_UNDEF == proc->rank) {
@@ -1389,7 +1393,7 @@ static pmix_status_t _hash_store_modex(pmix_gds_base_ctx_t ctx, pmix_proc_t *pro
         PMIX_DESTRUCT(&kv);
         /* continue along */
         PMIX_CONSTRUCT(&kv, pmix_kval_t);
-        rc = pmix_gds_base_modex_unpack_kval(key_fmt, pbkt, kmap, &kv);
+        PMIX_BFROPS_UNPACK(rc, pmix_globals.mypeer, pbkt, &kv, &cnt, PMIX_KVAL);
     }
     PMIX_DESTRUCT(&kv);
     if (PMIX_ERR_UNPACK_READ_PAST_END_OF_BUFFER != rc) {
@@ -1435,7 +1439,8 @@ static pmix_status_t nspace_del(const char *nspace)
     return PMIX_SUCCESS;
 }
 
-static pmix_status_t assemb_kvs_req(const pmix_proc_t *proc, pmix_list_t *kvs, pmix_buffer_t *buf,
+static pmix_status_t assemb_kvs_req(const pmix_proc_t *proc,
+                                    pmix_list_t *kvs, pmix_buffer_t *buf,
                                     void *cbdata)
 {
     pmix_status_t rc = PMIX_SUCCESS;
