@@ -12,7 +12,7 @@
  * Copyright (c) 2007      Sun Microsystems, Inc.  All rights reserved.
  * Copyright (c) 2009      Cisco Systems, Inc. All rights reserved.
  * Copyright (c) 2014-2020 Intel, Inc.  All rights reserved.
- * Copyright (c) 2021-2022 Nanook Consulting.  All rights reserved.
+ * Copyright (c) 2021-2025 Nanook Consulting  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -24,7 +24,7 @@
  * Send an email upon plog events.
  */
 
-#include "src/include/pmix_config.h"
+#include "pmix_config.h"
 #include "pmix_common.h"
 
 #include <stdarg.h>
@@ -171,7 +171,7 @@ static int send_email(char *msg)
     pmix_plog_smtp_component_t *c = &pmix_mca_plog_smtp_component;
 
     if (NULL == c->to_argv) {
-        c->to_argv = pmix_argv_split(c->to, ',');
+        c->to_argv = PMIx_Argv_split(c->to, ',');
         if (NULL == c->to_argv || NULL == c->to_argv[0]) {
             return PMIX_ERR_OUT_OF_RESOURCE;
         }
@@ -286,7 +286,7 @@ error:
 
         e = smtp_errno();
         smtp_strerror(e, em, sizeof(em));
-        pmix_show_help("help-pmix-plog-smtp.txt", "send_email failed", true,
+        pmix_show_help("help-pmix-plog.txt", "smtp:send_email failed", true,
                        "libesmtp library call failed", errmsg, em, e, msg);
     }
     return err;
