@@ -10,7 +10,7 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2015-2020 Intel, Inc.  All rights reserved.
- * Copyright (c) 2021-2024 Nanook Consulting  All rights reserved.
+ * Copyright (c) 2021-2025 Nanook Consulting  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -62,6 +62,19 @@ pmix_status_t pmix_ptl_base_set_peer(pmix_peer_t *peer, char *evar)
 
     vrs = getenv("PMIX_VERSION");
 
+    if (0 == strcmp(evar, "PMIX_SERVER_URI51")) {
+        /* we are talking to a v5.1 server */
+        PMIX_SET_PEER_TYPE(peer, PMIX_PROC_SERVER);
+        PMIX_SET_PEER_VERSION(peer, vrs, 5, 1);
+
+        pmix_output_verbose(2, pmix_ptl_base_framework.framework_output,
+                            "V51 SERVER DETECTED");
+
+        /* must use the v51 bfrops module */
+        PMIX_BFROPS_SET_MODULE(rc, pmix_globals.mypeer, peer, "v51");
+        return rc;
+    }
+
     if (0 == strcmp(evar, "PMIX_SERVER_URI41")) {
         /* we are talking to a v4.1 server */
         PMIX_SET_PEER_TYPE(peer, PMIX_PROC_SERVER);
@@ -134,6 +147,12 @@ pmix_status_t pmix_ptl_base_check_server_uris(pmix_peer_t *peer, char **ev)
     char *evar;
     pmix_status_t rc;
 
+    if (NULL != (evar = getenv("PMIX_SERVER_URI51"))) {
+        rc = pmix_ptl_base_set_peer(peer, "PMIX_SERVER_URI51");
+        *ev = evar;
+        return rc;
+    }
+
     if (NULL != (evar = getenv("PMIX_SERVER_URI41"))) {
         rc = pmix_ptl_base_set_peer(peer, "PMIX_SERVER_URI41");
         *ev = evar;
@@ -321,12 +340,20 @@ pmix_status_t pmix_ptl_base_parse_uri_file(char *filename,
             } while (retries < pmix_ptl_base.max_retries);
             /* otherwise, mark it as unreachable */
         }
+        if (!optional) {
+            pmix_show_help("help-ptl-base.txt", "file-not-found", true,
+                           filename, "could not be found");
+        }
         return PMIX_ERR_UNREACH;
     }
 
 process:
     fp = fopen(filename, "r");
     if (NULL == fp) {
+        if (!optional) {
+            pmix_show_help("help-ptl-base.txt", "file-not-found", true,
+                           filename, "could not be opened");
+        }
         return PMIX_ERR_UNREACH;
     }
     /* get the URI - might seem crazy, but there is actually
@@ -341,6 +368,7 @@ process:
         fclose(fp);
         tv.tv_sec = 0;
         tv.tv_usec = 10000; // use 0.01 sec as default
+        PMIX_CONSTRUCT_LOCK(&lock);
         pmix_event_evtimer_set(pmix_globals.evbase, &ev, timeout, &lock);
         PMIX_POST_OBJECT(&ev);
         pmix_event_evtimer_add(&ev, &tv);
@@ -352,7 +380,10 @@ process:
         }
     }
     if (NULL == srvr) {
-        PMIX_ERROR_LOG(PMIX_ERR_FILE_READ_FAILURE);
+        if (!optional) {
+            pmix_show_help("help-ptl-base.txt", "file-not-found", true,
+                           filename, "could not be read");
+        }
         fclose(fp);
         return PMIX_ERR_UNREACH;
     }
@@ -641,6 +672,10 @@ retry:
         }
         return rc;
     }
+    /* Assign the lower half of the tag space for sendrecvs */
+    peer->dyn_tags_start    = PMIX_PTL_TAG_DYNAMIC;
+    peer->dyn_tags_current  = PMIX_PTL_TAG_DYNAMIC;
+    peer->dyn_tags_end      = PMIX_PTL_TAG_DYNAMIC + (UINT32_MAX - PMIX_PTL_TAG_DYNAMIC)/2;
 
     return PMIX_SUCCESS;
 }
