@@ -7,7 +7,7 @@
  * Copyright (c) 2016      Mellanox Technologies, Inc.
  *                         All rights reserved.
  * Copyright (c) 2016      IBM Corporation.  All rights reserved.
- * Copyright (c) 2021-2022 Nanook Consulting.  All rights reserved.
+ * Copyright (c) 2021-2025 Nanook Consulting  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -728,12 +728,12 @@ void pmix_ptl_base_send_recv(int fd, short args, void *cbdata)
         return;
     }
 
-    /* take the next tag in the sequence */
-    pmix_ptl_base.current_tag++;
-    if (UINT32_MAX == pmix_ptl_base.current_tag) {
-        pmix_ptl_base.current_tag = PMIX_PTL_TAG_DYNAMIC;
+    /* take the next tag in the sequence of tags for this peer */
+    ms->peer->dyn_tags_current++;
+    if (ms->peer->dyn_tags_current == ms->peer->dyn_tags_end) {
+        ms->peer->dyn_tags_current = ms->peer->dyn_tags_start;
     }
-    tag = pmix_ptl_base.current_tag;
+    tag = ms->peer->dyn_tags_current;
 
     if (NULL != ms->cbfunc) {
         /* if a callback msg is expected, setup a recv for it */
@@ -835,7 +835,7 @@ void pmix_ptl_base_process_msg(int fd, short flags, void *cbdata)
                 pmix_output_verbose(5, pmix_ptl_base_framework.framework_output,
                                     "%s:%d EXECUTE CALLBACK for tag %u with %d bytes",
                                     pmix_globals.myid.nspace, pmix_globals.myid.rank,
-                                    msg->hdr.tag, (int)msg->hdr.nbytes);
+                                    msg->hdr.tag, (int)buf.bytes_used);
                 rcv->cbfunc(msg->peer, &msg->hdr, &buf, rcv->cbdata);
                 pmix_output_verbose(5, pmix_ptl_base_framework.framework_output,
                                     "%s:%d CALLBACK COMPLETE", pmix_globals.myid.nspace,
