@@ -17,7 +17,7 @@
  *                         and Technology (RIST). All rights reserved.
  * Copyright (c) 2018-2020 IBM Corporation.  All rights reserved.
  * Copyright (c) 2019      Mellanox Technologies, Inc. All rights reserved.
- * Copyright (c) 2021      Nanook Consulting.  All rights reserved.
+ * Copyright (c) 2021-2025 Nanook Consulting  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -32,9 +32,9 @@
  */
 
 #include "src/include/pmix_config.h"
-#include "include/pmix_common.h"
-#include "src/util/error.h"
-#include "src/util/output.h"
+#include "pmix_common.h"
+#include "src/util/pmix_error.h"
+#include "src/util/pmix_output.h"
 
 #include "src/mca/ptl/base/base.h"
 #include "src/mca/ptl/client/ptl_client.h"
@@ -45,17 +45,21 @@ static int component_query(pmix_mca_base_module_t **module, int *priority);
  * Instantiate the public struct with all of our public information
  * and pointers to our public functions in it
  */
-PMIX_EXPORT pmix_ptl_base_component_t mca_ptl_client_component
-    = {.base = {PMIX_PTL_BASE_VERSION_2_0_0,
+PMIX_EXPORT pmix_ptl_base_component_t pmix_mca_ptl_client_component = {
+    .base = {
+        PMIX_PTL_BASE_VERSION_2_0_0,
 
-                /* Component name and version */
-                .pmix_mca_component_name = "client",
-                PMIX_MCA_BASE_MAKE_VERSION(component, PMIX_MAJOR_VERSION, PMIX_MINOR_VERSION,
-                                           PMIX_RELEASE_VERSION),
+        /* Component name and version */
+        .pmix_mca_component_name = "client",
+        PMIX_MCA_BASE_MAKE_VERSION(component, PMIX_MAJOR_VERSION, PMIX_MINOR_VERSION,
+                                   PMIX_RELEASE_VERSION),
 
-                /* Component open and close functions */
-                .pmix_mca_query_component = component_query},
-       .priority = 50};
+        /* Component open and close functions */
+        .pmix_mca_query_component = component_query
+    },
+    .priority = 50
+};
+PMIX_MCA_BASE_COMPONENT_INIT(pmix, ptl, client)
 
 static int component_query(pmix_mca_base_module_t **module, int *priority)
 {
@@ -67,6 +71,6 @@ static int component_query(pmix_mca_base_module_t **module, int *priority)
     }
 
     *module = (pmix_mca_base_module_t *) &pmix_ptl_client_module;
-    *priority = mca_ptl_client_component.priority;
+    *priority = pmix_mca_ptl_client_component.priority;
     return PMIX_SUCCESS;
 }
