@@ -58,6 +58,8 @@ typedef struct {
     int nlocalprocs;
     pmix_info_t *info;
     size_t ninfo;
+    pmix_resource_unit_t *units;
+    size_t nunits;
     bool copied;
     char **keys;
     pmix_app_t *apps;
@@ -181,7 +183,6 @@ typedef struct {
     pmix_list_t gdata;  // cache of data given to me for passing to all clients
     char **genvars;     // argv array of envars given to me for passing to all clients
     pmix_list_t events; // list of pmix_regevents_info_t registered events
-    char **failedgrps;    // group IDs that failed to construct
     pmix_list_t iof;    // IO to be forwarded to clients
     pmix_list_t iof_residuals;  // leftover bytes waiting for newline
     pmix_list_t psets;  // list of known psets and memberships
@@ -190,6 +191,7 @@ typedef struct {
     char *tmpdir;             // temporary directory for this server
     char *system_tmpdir;      // system tmpdir
     bool fence_localonly_opt; // local-only fence optimization
+    pmix_list_t grp_collectives;  // group-op collectives
     // verbosity for server get operations
     int get_output;
     int get_verbose;
@@ -328,9 +330,8 @@ PMIX_EXPORT pmix_status_t pmix_server_iofstdin(pmix_peer_t *peer, pmix_buffer_t
 PMIX_EXPORT pmix_status_t pmix_server_iofdereg(pmix_peer_t *peer, pmix_buffer_t *buf,
                                                pmix_op_cbfunc_t cbfunc, void *cbdata);
 
-PMIX_EXPORT pmix_status_t pmix_server_grpconstruct(pmix_server_caddy_t *cd, pmix_buffer_t *buf);
-
-PMIX_EXPORT pmix_status_t pmix_server_grpdestruct(pmix_server_caddy_t *cd, pmix_buffer_t *buf);
+PMIX_EXPORT pmix_status_t pmix_server_group(pmix_server_caddy_t *cd, pmix_buffer_t *buf,
+                                            pmix_group_operation_t op);
 
 PMIX_EXPORT pmix_status_t pmix_server_event_recvd_from_client(pmix_peer_t *peer, pmix_buffer_t *buf,
                                                               pmix_op_cbfunc_t cbfunc,
@@ -366,6 +367,14 @@ PMIX_EXPORT pmix_status_t pmix_server_refresh_cache(pmix_server_caddy_t *cd,
                                                     pmix_buffer_t *buf,
                                                     pmix_op_cbfunc_t cbfunc);
 
+PMIX_EXPORT pmix_status_t pmix_server_resblk(pmix_server_caddy_t *cd,
+                                             pmix_buffer_t *buf,
+                                             pmix_op_cbfunc_t cbfunc);
+
+PMIX_EXPORT pmix_status_t pmix_server_session_ctrl(pmix_server_caddy_t *cd,
+                                                   pmix_buffer_t *buf,
+                                                   pmix_info_cbfunc_t cbfunc);
+
 PMIX_EXPORT pmix_status_t pmix_server_resolve_peers(pmix_server_caddy_t *cd,
                                                     pmix_buffer_t *buf,
                                                     pmix_info_cbfunc_t cbfunc);
@@ -399,5 +408,14 @@ static inline pmix_peer_t* pmix_get_peer_object(const pmix_proc_t *proc)
     return NULL;
 }
 
+// Utilities
+PMIX_EXPORT pmix_server_trkr_t *pmix_server_get_tracker(char *id, pmix_proc_t *procs,
+                                                        size_t nprocs, pmix_cmd_t type);
+
+PMIX_EXPORT pmix_server_trkr_t *pmix_server_new_tracker(char *id, pmix_proc_t *procs,
+                                                        size_t nprocs, pmix_cmd_t type);
+
+PMIX_EXPORT pmix_status_t pmix_server_collect_data(pmix_server_trkr_t *trk,
+                                                   pmix_buffer_t *buf);
 
 #endif // PMIX_SERVER_OPS_H
