@@ -15,7 +15,7 @@
  * Copyright (c) 2011      Oak Ridge National Labs.  All rights reserved.
  * Copyright (c) 2013-2020 Intel, Inc.  All rights reserved.
  * Copyright (c) 2015      Mellanox Technologies, Inc.  All rights reserved.
- * Copyright (c) 2021-2023 Nanook Consulting.  All rights reserved.
+ * Copyright (c) 2021-2025 Nanook Consulting  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -63,6 +63,7 @@ static struct option pctrlptions[] = {
     PMIX_OPTION_DEFINE(PMIX_CLI_URI, PMIX_ARG_REQD),
     PMIX_OPTION_DEFINE(PMIX_CLI_TMPDIR, PMIX_ARG_REQD),
 
+    PMIX_OPTION_DEFINE(PMIX_CLI_SIGNAL, PMIX_ARG_REQD),
     PMIX_OPTION_DEFINE(PMIX_CLI_REQ_ID, PMIX_ARG_REQD),
     PMIX_OPTION_DEFINE(PMIX_CLI_PAUSE, PMIX_ARG_NONE),
     PMIX_OPTION_DEFINE(PMIX_CLI_RESUME, PMIX_ARG_NONE),
@@ -127,19 +128,24 @@ int main(int argc, char **argv)
     pmix_tool_basename = "pctrl";
     gethostname(hostname, sizeof(hostname));
 
+    // setup the base infrastructure
+    if (PMIX_SUCCESS != pmix_init_util(NULL, 0, NULL)) {
+        return PMIX_ERROR;
+    }
+
     /* Parse the command line options */
     PMIX_CONSTRUCT(&results, pmix_cli_result_t);
     rc = pmix_cmd_line_parse(argv, pctrlshorts, pctrlptions,
                              NULL, &results, "help-pctrl.txt");
 
     if (PMIX_SUCCESS != rc) {
-        if (PMIX_ERR_SILENT == rc) {
-            exit(rc);
+        if (PMIX_OPERATION_SUCCEEDED == rc) {
+            exit(0);
         }
-        if (PMIX_OPERATION_SUCCEEDED != rc) {
+        if (PMIX_ERR_SILENT != rc) {
             fprintf(stderr, "%s: command line error (%s)\n", argv[0], PMIx_Error_string(rc));
-            exit(rc);
         }
+        exit(rc);
     }
 
     // handle relevant MCA params
@@ -151,11 +157,6 @@ int main(int argc, char **argv)
         }
     }
 
-    // setup the base infrastructure
-    if (PMIX_SUCCESS != pmix_init_util(NULL, 0, NULL)) {
-        return PMIX_ERROR;
-    }
-
     // check for common required command line option
     if (NULL == (opt = pmix_cmd_line_get_param(&results, PMIX_CLI_TARGETS))) {
         // must tell us the targets for the operation
