@@ -15,7 +15,7 @@
  * Copyright (c) 2016-2017 Los Alamos National Security, LLC. All rights
  *                         reserved.
  * Copyright (c) 2017      IBM Corporation. All rights reserved.
- * Copyright (c) 2021-2023 Nanook Consulting.  All rights reserved.
+ * Copyright (c) 2021-2025 Nanook Consulting  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -157,6 +157,21 @@ PMIX_CLASS_DECLARATION(pmix_cli_result_t);
 #define PMIX_CLI_TARGETS                "targets"                   // required
 #define PMIX_CLI_TERMINATE              "terminate"                 // none
 #define PMIX_CLI_PSET_NAME              "pset"                      // required
+#define PMIX_CLI_FWD_ENVAR              "x"                         // required
+#define PMIX_CLI_PREPEND_ENVAR          "prepend-env"               // required
+#define PMIX_CLI_APPEND_ENVAR           "append-env"                // required
+#define PMIX_CLI_UNSET_ENVAR            "unset-env"                 // required
+
+// Info options
+#define PMIX_CLI_INFO_ALL               "all"                       // none
+#define PMIX_CLI_INFO_ARCH              "arch"                      // none
+#define PMIX_CLI_INFO_CONFIG            "config"                    // none
+#define PMIX_CLI_INFO_HOSTNAME          "hostname"                  // none
+#define PMIX_CLI_INFO_INTERNAL          "internal"                  // none
+#define PMIX_CLI_INFO_PARAM             "param"                     // required
+#define PMIX_CLI_INFO_PATH              "path"                      // required
+#define PMIX_CLI_INFO_VERSION           "show-version"              // required
+
 
 typedef void (*pmix_cmd_line_store_fn_t)(const char *name, const char *option,
                                          pmix_cli_result_t *results);
