@@ -12,8 +12,8 @@
  * Copyright (c) 2016      Mellanox Technologies, Inc.
  *                         All rights reserved.
  * Copyright (c) 2016      IBM Corporation.  All rights reserved.
- * Copyright (c) 2021-2023 Nanook Consulting.  All rights reserved.
- * Copyright (c) 2022-2023 Triad National Security, LLC. All rights reserved.
+ * Copyright (c) 2021-2024 Nanook Consulting  All rights reserved.
+ * Copyright (c) 2022-2024 Triad National Security, LLC. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -22,6 +22,9 @@
  *
  */
 
+// TODO(skg) Assert that the tma from the table and keyindex match in all
+// relevant places.
+
 #include "src/include/pmix_config.h"
 
 #include "src/include/pmix_hash_string.h"
@@ -42,6 +45,15 @@
 
 #include "src/util/pmix_hash.h"
 
+/**
+ * Utility function that takes a pointer to a pmix_keyindex_t and returns the
+ * global keyindex if NULL, returns the input otherwise.
+ */
+static inline pmix_keyindex_t *get_keyindex_ptr(pmix_keyindex_t *input)
+{
+    return input ? input : &pmix_globals.keyindex;
+}
+
 /**
  * Data for a particular pmix process
  * The name association is maintained in the
@@ -99,10 +111,13 @@ static void pddes(pmix_proc_data_t *p)
     }
     PMIX_RELEASE(p->quals);
 }
-static PMIX_CLASS_INSTANCE(pmix_proc_data_t, pmix_object_t, pdcon, pddes);
+static PMIX_CLASS_INSTANCE(pmix_proc_data_t,
+                           pmix_object_t,
+                           pdcon, pddes);
 
 static pmix_dstor_t *lookup_keyval(pmix_proc_data_t *proc, uint32_t kid,
-                                   pmix_info_t *qualifiers, size_t nquals);
+                                   pmix_info_t *qualifiers, size_t nquals,
+                                   pmix_keyindex_t *kidx);
 static pmix_proc_data_t *lookup_proc(pmix_hash_table_t *jtable, uint32_t id, bool create);
 static void erase_qualifiers(pmix_proc_data_t *proc,
                              uint32_t index);
@@ -110,7 +125,8 @@ static void erase_qualifiers(pmix_proc_data_t *proc,
 
 pmix_status_t pmix_hash_store(pmix_hash_table_t *table,
                               pmix_rank_t rank, pmix_kval_t *kin,
-                              pmix_info_t *qualifiers, size_t nquals)
+                              pmix_info_t *qualifiers, size_t nquals,
+                              pmix_keyindex_t *kidx)
 {
     pmix_proc_data_t *proc_data;
     uint32_t kid;
@@ -121,11 +137,13 @@ pmix_status_t pmix_hash_store(pmix_hash_table_t *table,
     pmix_qual_t *qarray;
     size_t n, m = 0;
     pmix_tma_t *const tma = pmix_obj_get_tma(&table->super);
+    pmix_keyindex_t *const keyindex = get_keyindex_ptr(kidx);
 
     pmix_output_verbose(10, pmix_gds_base_framework.framework_output,
-                        "HASH:STORE:QUAL rank %s key %s",
-                        PMIX_RANK_PRINT(rank),
-                        (NULL == kin) ? "NULL KVAL" : kin->key);
+                        "%s HASH:STORE:QUAL table %s rank %s key %s",
+                        PMIX_NAME_PRINT(&pmix_globals.myid),
+                        (NULL == table->ht_label) ? "UNKNOWN" : table->ht_label,
+                        PMIX_RANK_PRINT(rank), (NULL == kin) ? "NULL KVAL" : kin->key);
 
     if (PMIX_UNLIKELY(NULL == kin)) {
         return PMIX_ERR_BAD_PARAM;
@@ -135,7 +153,7 @@ pmix_status_t pmix_hash_store(pmix_hash_table_t *table,
      * moved to the periphery of the PMIx library so we can
      * refer to the key numerically throughout the internals
      */
-    p = pmix_hash_lookup_key(UINT32_MAX, kin->key);
+    p = pmix_hash_lookup_key(UINT32_MAX, kin->key, keyindex);
     if (PMIX_UNLIKELY(NULL == p)) {
         /* we don't know this key */
         pmix_output_verbose(10, pmix_gds_base_framework.framework_output,
@@ -153,7 +171,7 @@ pmix_status_t pmix_hash_store(pmix_hash_table_t *table,
     }
 
     /* see if we already have this key-value */
-    hv = lookup_keyval(proc_data, kid, qualifiers, nquals);
+    hv = lookup_keyval(proc_data, kid, qualifiers, nquals, keyindex);
     if (NULL != hv) {
         if (PMIX_UNLIKELY(9 < pmix_output_get_verbosity(pmix_gds_base_framework.framework_output))) {
             // Note that this doesn't have to use a TMA because it is just a
@@ -177,8 +195,8 @@ pmix_status_t pmix_hash_store(pmix_hash_table_t *table,
                 // temporary value.
                 char *tmp;
                 tmp = PMIx_Value_string(kin->value);
-                pmix_output(0, "%s VALUE UPDATING TO: %s",
-                            PMIX_NAME_PRINT(&pmix_globals.myid), tmp);
+                pmix_output(0, "%s KEY %s VALUE UPDATING TO: %s",
+                            PMIX_NAME_PRINT(&pmix_globals.myid), kin->key, tmp);
                 free(tmp);
             }
             pmix_bfrops_base_tma_value_release(&hv->value, tma);
@@ -212,7 +230,7 @@ pmix_status_t pmix_hash_store(pmix_hash_table_t *table,
             qarray = (pmix_qual_t*)darray->array;
             for (n=0, m=0; n < nquals; n++) {
                 if (PMIX_INFO_IS_QUALIFIER(&qualifiers[n])) {
-                    p = pmix_hash_lookup_key(UINT32_MAX, qualifiers[n].key);
+                    p = pmix_hash_lookup_key(UINT32_MAX, qualifiers[n].key, keyindex);
                     if (PMIX_UNLIKELY(NULL == p)) {
                         /* we don't know this key */
                         pmix_output_verbose(10, pmix_gds_base_framework.framework_output,
@@ -256,13 +274,60 @@ pmix_status_t pmix_hash_store(pmix_hash_table_t *table,
                     PMIX_NAME_PRINT(&pmix_globals.myid),
                     kin->key, v,
                     PMIX_RANK_PRINT(rank), (unsigned)m,
-                    table->ht_label);
+                    (NULL == table->ht_label) ? "UNKNOWN" : table->ht_label);
         free(v);
     }
     pmix_pointer_array_add(proc_data->data, hv);
     return PMIX_SUCCESS;
 }
 
+static pmix_status_t make_copy(pmix_regattr_input_t *p,
+                               pmix_dstor_t *hv,
+                               pmix_list_t *kvals,
+                               pmix_proc_data_t *proc_data,
+                               pmix_keyindex_t *const keyindex)
+{
+    pmix_kval_t *kv;
+    pmix_data_array_t *darray;
+    pmix_qual_t *quals;
+    pmix_info_t *iptr;
+    size_t nq, m;
+
+    if (UINT32_MAX != hv->qualindex) {
+        /* this is a qualified value - need to return it as such */
+        PMIX_KVAL_NEW(kv, PMIX_QUALIFIED_VALUE);
+        darray = (pmix_data_array_t*)pmix_pointer_array_get_item(proc_data->quals, hv->qualindex);
+        quals = (pmix_qual_t*)darray->array;
+        nq = darray->size;
+        PMIX_DATA_ARRAY_CREATE(darray, nq+1, PMIX_INFO);
+        iptr = (pmix_info_t*)darray->array;
+        /* the first location is the actual value */
+        PMIX_LOAD_KEY(iptr[0].key, p->string);
+        PMIx_Value_xfer(&iptr[0].value, hv->value);
+        /* now add the qualifiers */
+        for (m=0; m < nq; m++) {
+            p = pmix_hash_lookup_key(quals[m].index, NULL, keyindex);
+            if (NULL == p) {
+                /* should never happen */
+                PMIX_RELEASE(kv);
+                PMIX_DATA_ARRAY_FREE(darray);
+                return PMIX_ERR_BAD_PARAM;
+            }
+            PMIX_LOAD_KEY(iptr[m+1].key, p->string);
+            PMIx_Value_xfer(&iptr[m+1].value, quals[m].value);
+            PMIX_INFO_SET_QUALIFIER(&iptr[m+1]);
+        }
+        kv->value->type = PMIX_DATA_ARRAY;
+        kv->value->data.darray = darray;
+        pmix_list_append(kvals, &kv->super);
+    } else {
+        PMIX_KVAL_NEW(kv, p->string);
+        PMIx_Value_xfer(kv->value, hv->value);
+        pmix_list_append(kvals, &kv->super);
+    }
+    return PMIX_SUCCESS;
+}
+
 // TODO(skg) We may have to provide a different fetch entry point with different
 // semantics for gds shmem to further reduce memory usage. For now this seems to
 // work for our current use case.
@@ -270,7 +335,8 @@ pmix_status_t pmix_hash_fetch(pmix_hash_table_t *table,
                               pmix_rank_t rank,
                               const char *key,
                               pmix_info_t *qualifiers, size_t nquals,
-                              pmix_list_t *kvals)
+                              pmix_list_t *kvals,
+                              pmix_keyindex_t *kidx)
 {
     pmix_status_t rc;
     pmix_proc_data_t *proc_data;
@@ -278,19 +344,15 @@ pmix_status_t pmix_hash_fetch(pmix_hash_table_t *table,
     uint32_t id, kid=UINT32_MAX;
     char *node;
     pmix_regattr_input_t *p;
-    pmix_info_t *iptr;
-    size_t nq, m;
     int n;
-    pmix_kval_t *kv;
     bool fullsearch = false;
-    pmix_data_array_t *darray;
-    pmix_qual_t *quals;
+    pmix_keyindex_t *const keyindex = get_keyindex_ptr(kidx);
 
     pmix_output_verbose(10, pmix_gds_base_framework.framework_output,
-                        "%s HASH:FETCH id %s key %s",
+                        "%s HASH:FETCH table %s id %s key %s",
                         PMIX_NAME_PRINT(&pmix_globals.myid),
-                        PMIX_RANK_PRINT(rank),
-                        (NULL == key) ? "NULL" : key);
+                        (NULL == table->ht_label) ? "UNKNOWN" : table->ht_label,
+                        PMIX_RANK_PRINT(rank), (NULL == key) ? "NULL" : key);
 
     /* - PMIX_RANK_UNDEF should return following statuses
      *     PMIX_ERR_NOT_FOUND | PMIX_SUCCESS
@@ -316,7 +378,7 @@ pmix_status_t pmix_hash_fetch(pmix_hash_table_t *table,
          * moved to the periphery of the PMIx library so we can
          * refer to the key numerically throughout the internals
          */
-        p = pmix_hash_lookup_key(UINT32_MAX, key);
+        p = pmix_hash_lookup_key(UINT32_MAX, key, keyindex);
         if (NULL == p) {
             /* we don't know this key */
             return PMIX_ERR_BAD_PARAM;
@@ -342,7 +404,7 @@ pmix_status_t pmix_hash_fetch(pmix_hash_table_t *table,
             for (n=0; n < proc_data->data->size; n++) {
                 hv = (pmix_dstor_t*)pmix_pointer_array_get_item(proc_data->data, n);
                 if (NULL != hv) {
-                    p = pmix_hash_lookup_key(hv->index, NULL);
+                    p = pmix_hash_lookup_key(hv->index, NULL, keyindex);
                     if (NULL == p) {
                         return PMIX_ERR_NOT_FOUND;
                     }
@@ -354,53 +416,24 @@ pmix_status_t pmix_hash_fetch(pmix_hash_table_t *table,
                         PMIX_CHECK_RESERVED_KEY(p->string)) {
                         continue;
                     }
-                    if (UINT32_MAX != hv->qualindex) {
-                        pmix_output_verbose(10, pmix_gds_base_framework.framework_output,
-                                            "%s INCLUDE %s VALUE %u FROM TABLE %s FOR RANK %s",
-                                            PMIX_NAME_PRINT(&pmix_globals.myid), p->name,
-                                            (unsigned)hv->value->data.size, table->ht_label, PMIX_RANK_PRINT(rank));
-                        /* this is a qualified value - need to return it as such */
-                        PMIX_KVAL_NEW(kv, PMIX_QUALIFIED_VALUE);
-                        darray = (pmix_data_array_t*)pmix_pointer_array_get_item(proc_data->quals, hv->qualindex);
-                        quals = (pmix_qual_t*)darray->array;
-                        nq = darray->size;
-                        PMIX_DATA_ARRAY_CREATE(darray, nq+1, PMIX_INFO);
-                        iptr = (pmix_info_t*)darray->array;
-                        /* the first location is the actual value */
-                        PMIX_LOAD_KEY(iptr[0].key, p->string);
-                        PMIx_Value_xfer(&iptr[0].value, hv->value);
-                        /* now add the qualifiers */
-                        for (m=0; m < nq; m++) {
-                            p = pmix_hash_lookup_key(quals[m].index, NULL);
-                            if (NULL == p) {
-                                /* should never happen */
-                                PMIX_RELEASE(kv);
-                                PMIX_DATA_ARRAY_FREE(darray);
-                                return PMIX_ERR_BAD_PARAM;
-                            }
-                            PMIX_LOAD_KEY(iptr[m+1].key, p->string);
-                            PMIx_Value_xfer(&iptr[m+1].value, quals[m].value);
-                            PMIX_INFO_SET_QUALIFIER(&iptr[m+1]);
-                        }
-                        kv->value->type = PMIX_DATA_ARRAY;
-                        kv->value->data.darray = darray;
-                        pmix_list_append(kvals, &kv->super);
-                    } else {
-                        PMIX_KVAL_NEW(kv, p->string);
-                        PMIx_Value_xfer(kv->value, hv->value);
-                        pmix_list_append(kvals, &kv->super);
+                    pmix_output_verbose(10, pmix_gds_base_framework.framework_output,
+                                        "%s INCLUDE %s VALUE %s FROM TABLE %s FOR RANK %s",
+                                        PMIX_NAME_PRINT(&pmix_globals.myid), p->name,
+                                        PMIx_Value_string(hv->value),
+                                        (NULL == table->ht_label) ? "UNKNOWN" : table->ht_label,
+                                        PMIX_RANK_PRINT(rank));
+                    rc = make_copy(p, hv, kvals, proc_data, keyindex);
+                    if (PMIX_UNLIKELY(PMIX_SUCCESS != rc)) {
+                        return rc;
                     }
                 }
             }
             return PMIX_SUCCESS;
         } else {
             /* find the value from within this data object */
-            hv = lookup_keyval(proc_data, kid, qualifiers, nquals);
+            hv = lookup_keyval(proc_data, kid, qualifiers, nquals, keyindex);
             if (NULL != hv) {
-                /* create the copy */
-                PMIX_KVAL_NEW(kv, key);
-                PMIx_Value_xfer(kv->value, hv->value);
-                pmix_list_append(kvals, &kv->super);
+                rc = make_copy(p, hv, kvals, proc_data, keyindex);
                 break;
             } else if (!fullsearch) {
                 pmix_output_verbose(10, pmix_gds_base_framework.framework_output,
@@ -423,7 +456,8 @@ pmix_status_t pmix_hash_fetch(pmix_hash_table_t *table,
 }
 
 pmix_status_t pmix_hash_remove_data(pmix_hash_table_t *table,
-                                    pmix_rank_t rank, const char *key)
+                                    pmix_rank_t rank, const char *key,
+                                    pmix_keyindex_t *kidx)
 {
     pmix_status_t rc = PMIX_SUCCESS;
     pmix_proc_data_t *proc_data;
@@ -433,9 +467,10 @@ pmix_status_t pmix_hash_remove_data(pmix_hash_table_t *table,
     char *node;
     pmix_regattr_input_t *p;
     pmix_tma_t *const tma = pmix_obj_get_tma(&table->super);
+    pmix_keyindex_t *const keyindex = get_keyindex_ptr(kidx);
 
     if (NULL != key) {
-        p = pmix_hash_lookup_key(UINT32_MAX, key);
+        p = pmix_hash_lookup_key(UINT32_MAX, key, keyindex);
         if (PMIX_UNLIKELY(NULL == p)) {
             /* we don't know this key */
             return PMIX_ERR_BAD_PARAM;
@@ -526,7 +561,8 @@ pmix_status_t pmix_hash_remove_data(pmix_hash_table_t *table,
  * Find data for a given key in a given pmix_list_t.
  */
 static pmix_dstor_t *lookup_keyval(pmix_proc_data_t *proc_data, uint32_t kid,
-                                   pmix_info_t *qualifiers, size_t nquals)
+                                   pmix_info_t *qualifiers, size_t nquals,
+                                   pmix_keyindex_t *kidx)
 {
     pmix_dstor_t *d;
     pmix_data_array_t *darray;
@@ -534,8 +570,9 @@ static pmix_dstor_t *lookup_keyval(pmix_proc_data_t *proc_data, uint32_t kid,
     pmix_regattr_input_t *p;
     size_t m, numquals = 0, nq, nfound;
     int n;
+    pmix_keyindex_t *const keyindex = get_keyindex_ptr(kidx);
 
-    p = pmix_hash_lookup_key(kid, NULL);
+    p = pmix_hash_lookup_key(kid, NULL, keyindex);
 
     if (NULL != qualifiers) {
         /* count the qualifiers */
@@ -566,7 +603,7 @@ static pmix_dstor_t *lookup_keyval(pmix_proc_data_t *proc_data, uint32_t kid,
                     if (!PMIX_INFO_IS_QUALIFIER(&qualifiers[m])) {
                         continue;
                     }
-                    p = pmix_hash_lookup_key(UINT32_MAX, qualifiers[m].key);
+                    p = pmix_hash_lookup_key(UINT32_MAX, qualifiers[m].key, keyindex);
                     if (NULL == p) {
                         /* we don't know this key */
                         return NULL;
@@ -624,62 +661,54 @@ static pmix_proc_data_t *lookup_proc(pmix_hash_table_t *jtable, uint32_t id, boo
 }
 
 void pmix_hash_register_key(uint32_t inid,
-                            pmix_regattr_input_t *ptr)
+                            pmix_regattr_input_t *ptr,
+                            pmix_keyindex_t *kidx)
 {
     pmix_regattr_input_t *p = NULL;
+    pmix_keyindex_t *const keyindex = get_keyindex_ptr(kidx);
 
     if (UINT32_MAX == inid) {
         /* store the pointer in the array */
-        pmix_pointer_array_set_item(&pmix_globals.keyindex, pmix_globals.next_keyid, ptr);
-        ptr->index = pmix_globals.next_keyid;
-        pmix_globals.next_keyid += 1;
+        pmix_pointer_array_set_item(keyindex->table, (int)keyindex->next_id, ptr);
+        ptr->index = keyindex->next_id;
+        keyindex->next_id += 1;
         return;
     }
 
     /* check to see if this key was already registered */
-    p = pmix_pointer_array_get_item(&pmix_globals.keyindex, inid);
+    p = pmix_pointer_array_get_item(keyindex->table, inid);
     if (NULL != p) {
         /* already have this one */
         return;
     }
     /* store the pointer in the table */
-    pmix_pointer_array_set_item(&pmix_globals.keyindex, inid, ptr);
+    pmix_pointer_array_set_item(keyindex->table, inid, ptr);
 }
 
-// TODO(skg) We may have to add a TMA wrapper for this call.
+// skg: Note that one may have to add a TMA wrapper for this call if changes are
+// made to how pmix_hash operates. Something for developers to keep in mind.
 pmix_regattr_input_t* pmix_hash_lookup_key(uint32_t inid,
-                                           const char *key)
+                                           const char *key,
+                                           pmix_keyindex_t *kidx)
 {
     int id;
     pmix_regattr_input_t *ptr = NULL;
+    pmix_keyindex_t *const keyindex = get_keyindex_ptr(kidx);
 
     if (UINT32_MAX == inid) {
         if (NULL == key) {
             /* they have to give us something! */
             return NULL;
         }
-        if (PMIX_CHECK_RESERVED_KEY(key)) {
-            /* reserved keys are in the front of the table */
-            for (id = 0; id < PMIX_INDEX_BOUNDARY; id++) {
-                ptr = pmix_pointer_array_get_item(&pmix_globals.keyindex, id);
-                if (NULL != ptr) {
-                    if (0 == strcmp(key, ptr->string)) {
-                        return ptr;
-                    }
-                }
-            }
-            /* reserved keys must already have been registered */
-            return NULL;
-        }
-        /* unreserved keys are at the back of the table */
-        for (id = PMIX_INDEX_BOUNDARY; id < pmix_globals.keyindex.size; id++) {
-            ptr = pmix_pointer_array_get_item(&pmix_globals.keyindex, id);
+        for (id = 0; id < keyindex->table->size; id++) {
+            ptr = pmix_pointer_array_get_item(keyindex->table, id);
             if (NULL != ptr) {
                 if (0 == strcmp(key, ptr->string)) {
                     return ptr;
                 }
             }
         }
+
         /* we didn't find it - register it */
         ptr = (pmix_regattr_input_t*)pmix_malloc(sizeof(pmix_regattr_input_t));
         ptr->name = strdup(key);
@@ -688,7 +717,7 @@ pmix_regattr_input_t* pmix_hash_lookup_key(uint32_t inid,
         ptr->description = (char**)pmix_malloc(2 * sizeof(char*));
         ptr->description[0] = strdup("USER DEFINED");
         ptr->description[1] = NULL;
-        pmix_hash_register_key(UINT32_MAX, ptr);
+        pmix_hash_register_key(UINT32_MAX, ptr, keyindex);
         return ptr;
     }
 
@@ -697,7 +726,7 @@ pmix_regattr_input_t* pmix_hash_lookup_key(uint32_t inid,
      * non-reserved key, then it had to be registered or else the caller
      * would not have an index to pass us. Thus, the pointer is either
      * found or not - we don't register it if not found. */
-    ptr = pmix_pointer_array_get_item(&pmix_globals.keyindex, inid);
+    ptr = pmix_pointer_array_get_item(keyindex->table, inid);
     return ptr;
 }
 
